// MCEDataEngine.cpp: implementation of the CMCEDataEngine class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

// Include the OpenGL headers
#include "gl\gl.h"
#include "gl\glu.h"

#include <float.h> // float number limits
#include <math.h>  // matematic functions

#include "MCE.h"
#include "MCEDataEngine.h"

// support loging function
#include "log.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMCEDataEngine::CMCEDataEngine():
m_bDataExist(false),m_bNeedRecalculateMap(true)
{
	LoadInitialValues();
}

CMCEDataEngine::~CMCEDataEngine()
{

}

BOOL CMCEDataEngine::ReadFouFile(CString filename, int map_number)
{

	CString strInputData;
	ELD_MapData	*pMapData;
	MoleculeData *pMolecule;

	// mark the map as initialized
	MolData[map_number].bInitialized=TRUE;

	pMapData = &MolData[map_number].MapData;
	pMolecule = &MolData[map_number].Molecule;

	pMapData->strMapFileName= filename;

	// support variable
	int nLineNumber=0;
	int i, j;
	int x,y,z;
	
	TRACE("Reading fou file: %s\n",pMapData->strMapFileName);
	LogMsg(pMapData->strMapFileName);
	LogMsg("\n");

	CFile f( pMapData->strMapFileName, CFile::modeRead);

	CArchive ar(&f,CArchive::load);


	/* for eventual debugging purpose
	while (ar.ReadString(strInputData))
	{
		nLineNumber++;
		TRACE("Line: %d Content: %s\n",nLineNumber,strInputData);
		//LogMsg(strInputData);
		//LogMsg("\n");
	}
	return FALSE;
	*/

	ar.ReadString(strInputData); //INFO DOWN, ....?
	ar.ReadString(strInputData); //TRAN ?

	for (i=0;i<3;i++)
	{
		for(j=0;j<3;j++)
		{
			ar.ReadString(strInputData);
			pMapData->PlaneMatrix[i][j]=(float) atof(strInputData);
			TRACE( "%d %d %f \n",i,j,pMapData->PlaneMatrix[i][j]);
		}
	}

	for (i=0;i<3;i++)
	{
		ar.ReadString(strInputData);
		pMapData->PlaneTranslation[i]=(float) atof(strInputData);
	}

	ar.ReadString(strInputData); // CELL

	for (i=0;i<6;i++)
	{
		ar.ReadString(strInputData);
		pMapData->Cell[i]=(float) atof(strInputData);
	}

	ar.ReadString(strInputData); // L14

	for (i=0;i<3;i++)
	{
		ar.ReadString(strInputData);
		pMapData->AxisParam[i].Start=(float) atof(strInputData);

		ar.ReadString(strInputData);
		pMapData->AxisParam[i].Step=(float) atof(strInputData);

		ar.ReadString(strInputData);
		pMapData->AxisParam[i].End=(float) atof(strInputData);

		ar.ReadString(strInputData);
		pMapData->AxisParam[i].Division=(float) atof(strInputData);
	}

	ar.ReadString(strInputData); // SIZE

	ar.ReadString(strInputData);
	pMapData->Lines=atoi(strInputData);

	ar.ReadString(strInputData);
	pMapData->Columns=atoi(strInputData);

	ar.ReadString(strInputData);
	pMapData->Sections=atoi(strInputData);

	// !!! THis part of code is not memory-optimal !!!
	// ReDim Voxel(MapData.Lines, MapData.Columns, MapData.Sections)

	for (z=0;z<pMapData->Sections;z++)
	{
		ar.ReadString(strInputData); // BLOCK
		ar.ReadString(strInputData); // N

		for(x=0;x<pMapData->Lines;x++)
		{
			for(y=0;y<pMapData->Columns;y++)
			{
				ar.ReadString(strInputData);
				pMapData->Voxel[x][y][z]=(float) atof(strInputData);
			}
		}
	}

	TRACE("Finished reading Voxel Map");

		// mlecular data input

	ar.ReadString(strInputData); //LIST5

	ar.ReadString(strInputData);
	pMolecule->Natoms=atoi(strInputData);

	if (pMolecule->Natoms>0)
	{
		ar.ReadString(strInputData);
		int atom_record_lenght=atoi(strInputData);
		for (i=0;i<pMolecule->Natoms;i++)
		{
			ar.ReadString(strInputData); // atom name
			strncpy(pMolecule->a[i].name,strInputData,2);

			ar.ReadString(strInputData);
			pMolecule->a[i].atm_label_number=atoi(strInputData);

			ar.ReadString(strInputData);
			pMolecule->a[i].occ=(float) atof(strInputData);

			ar.ReadString(strInputData); // reserved, dummy now

			ar.ReadString(strInputData);
			pMolecule->a[i].fc.x=(float) atof(strInputData);

			ar.ReadString(strInputData);
			pMolecule->a[i].fc.y=(float) atof(strInputData);

			ar.ReadString(strInputData);
			pMolecule->a[i].fc.z=(float) atof(strInputData);
			
			ar.ReadString(strInputData);
			pMolecule->a[i].uiso=(float) atof(strInputData);
			
			for(j=0;j<6;j++)
			{
				ar.ReadString(strInputData);
				pMolecule->a[i].u[j]=(float) atof(strInputData);
			}

			// dumy data from future fou versions
			if (atom_record_lenght>14)
			{
				for(j=15;j<=atom_record_lenght;j++)
				{
					ar.ReadString(strInputData); // dummy reading
				}
			}
		}// next j
	} // endif atom presents

	TRACE("Finished reading Molecule data");

	ar.Close();
	f.Close();

	m_bDataExist=TRUE;
	m_bNeedRecalculateMap=TRUE;


	// the rest of nessesery data initialization ...
	CMCEDataEngine::SetLayerLimitsFromVoxel(pMapData);
	CMCEDataEngine::StuctureToCart(pMolecule,pMapData);
	CMCEDataEngine::SetupConectivity(pMolecule);
	// fill the end lattice end points field by actual vaslues
	CMCEDataEngine::GenerateLatticePoint(pMapData);

	return TRUE;
}

void CMCEDataEngine::Scene()
{
	if(m_bDataExist)
	{
		if(m_bNeedRecalculateMap)
		{
			CMCEDataEngine::OnRecalulateMap();
			m_bNeedRecalculateMap=FALSE;
		}
			CMCEDataEngine::RenderScene();	
	}
}

void CMCEDataEngine::RenderScene()
{	
	glEnable(GL_LIGHTING);
		DrawMark();
		DrawPeaks();
	glDisable(GL_LIGHTING);

	if(Setup.bMoleculeVisible)
	{
		DrawBonds();
		glEnable(GL_LIGHTING);
			DrawAtoms();
		glDisable(GL_LIGHTING);
	}

	if(Setup.bUnitCellVisible)
	{
		DrawUnitCell();
	}

	//3D map
	glCallList(MCE_CONTOURS);

	// 2D map
	glCallList(MCE_CONTOURS2D);
}



void CMCEDataEngine::OnRecalulateMap() 
{
//	CMCEDataEngine::GenerateTrianglesFromVoxels();
	CMCEDataEngine::GenerateContoursFromVoxel();
	CMCEDataEngine::GenerateDisplayLists();
}

void CMCEDataEngine::SetLayerLimitsFromVoxel(ELD_MapData *pMapData)
{
int x;
	int y;
	int z;
	int xmax;
	int ymax;
	int zmax;
	float value;

    // reset global map variables
	float maximum=FLT_MIN;
	float minimum=FLT_MAX;

	xmax=pMapData->Lines; //X
	ymax=pMapData->Columns; //Y
	zmax=pMapData->Sections; //Z

	for (x=0;x<xmax;x++)
	{
		for (y=0;y<ymax;y++)
		{
			for (z=0;z<zmax;z++)
			{
				value=pMapData->Voxel[x][y][z];
				if (value >= maximum) maximum = value;
				if (value <= minimum) minimum = value;
			}
		}
	}

	pMapData->layer_limits[0]=(float)(minimum+0.1*(maximum-minimum));
	pMapData->layer_limits[1]=(float)(minimum+0.4*(maximum-minimum));
	pMapData->layer_limits[2]=(float)(minimum+0.7*(maximum-minimum));

	pMapData->maximum=maximum;
	pMapData->minimum=minimum;

	pMapData->layer_visibility[0]=TRUE;
	pMapData->layer_visibility[1]=TRUE;
	pMapData->layer_visibility[2]=TRUE;

	// 2D map parameters
	pMapData->map_2D_level=0;
	pMapData->map_2D_visibility=TRUE;
}

void CMCEDataEngine::GenerateContoursFromVoxel()
{
	// do it for all maps we have
	for (int i=0;i<NMAPS;i++)
	{
		// initialize the Mesh values

		M[i].iNcontourlines[0]=0;
		M[i].iNcontourlines[1]=0;
		M[i].iNcontourlines[2]=0;

		
		M2D[i].iNcontourlines[0]=0;
		M2D[i].iNcontourlines[1]=0;
		M2D[i].iNcontourlines[2]=0;

		if(MolData[i].bInitialized)
		{
			// 3D maps
			// slight speed optimization (compute 3D map only if any layer is visible)
			if(MolData[i].MapData.layer_visibility[0]
			 | MolData[i].MapData.layer_visibility[1]
			 | MolData[i].MapData.layer_visibility[2])
			{
				CMCEDataEngine::VoxelDirectionalContour(&MolData[i].MapData,X,&M[i]);
				CMCEDataEngine::VoxelDirectionalContour(&MolData[i].MapData,Y,&M[i]);
				CMCEDataEngine::VoxelDirectionalContour(&MolData[i].MapData,Z,&M[i]);	
			}

			// 2D maps - easy to do, compute always
			CMCEDataEngine::VoxelDirectionalContour2D(&MolData[i].MapData,Z,&M2D[i]);

		}
	}
}

// 3D map conotour 

void CMCEDataEngine::VoxelDirectionalContour(ELD_MapData *pMapData,contour_direction direction,LineMesh *pM)
{
	int x;
	int y;
	int z;
	int xmax;
	int ymax;
	int zmax;
	int i,j,k;

	int iNewLines;
	GRIDCELL2D grid;
	LINE2D newlines[2];

	xmax=pMapData->Lines; //X
	ymax=pMapData->Columns; //Y
	zmax=pMapData->Sections; //Z
	
	switch(direction)
	{
	case X:// map in X direction, x,y of the 2D grid coresponds to 3D z,y
		{ 
		 for (x=0;x<xmax;x++)
		 {
	      for (y=0;y<(ymax-1);y++)
		  {
	       for (z=0;z<(zmax-1);z++)
		   { // prepare the grid values		
		   grid.p[0].x=(float)z;
		   grid.p[0].y=(float)y;
		   grid.val[0]=pMapData->Voxel[x][y][z];
	
		   grid.p[1].x=(float)z+1;
		   grid.p[1].y=(float)y;
		   grid.val[1]=pMapData->Voxel[x][y][z+1];

		   grid.p[2].x=(float)z+1;
		   grid.p[2].y=(float)y+1;
		   grid.val[2]=pMapData->Voxel[x][y+1][z+1];

		   grid.p[3].x=(float)z;
		   grid.p[3].y=(float)y+1;
		   grid.val[3]=pMapData->Voxel[x][y+1][z];
				
		   for (i=0;i<=2;i++)
		   {
		    if ((pM->iNcontourlines[i]+2)<MAXCONTOURLINES) // check the max avaaliable number of lines
			{
			 iNewLines= Polygonise2D(grid,double(pMapData->layer_limits[i]),&newlines[0]);
			 // copy lines from the buffer to destination
			 for (j=0;j<iNewLines;j++)
			 {
			  for(k=0;k<=1;k++) // copy the begining and end of the line to destination
			  {
				pM->contourlines[i][pM->iNcontourlines[i]+j].p[k]=
				IndToCoord3Df((float)x,newlines[j].p[k].y,newlines[j].p[k].x,pMapData);
			  }// next k
			 }// next j
			 pM->iNcontourlines[i]=pM->iNcontourlines[i]+iNewLines;
			}// endif
			else
			{
				AfxMessageBox("iNcontourlines[i]+2)>MAXCONTOURLINES !!");
			}

		   }// next i							
		  }// next z
		 }// next y
		}// next x
		}// end case
	break;

	case Y:// map in Y direction, x,y of the 2D grid coresponds to 3D x,z
		{
		 for (y=0;y<ymax;y++)
		 {
	      for (x=0;x<(xmax-1);x++)
		  {
	       for (z=0;z<(zmax-1);z++)
		   { // prepare the grid values		
		   grid.p[0].x=(float)x;
		   grid.p[0].y=(float)z;
		   grid.val[0]=pMapData->Voxel[x][y][z];

		   grid.p[1].x=(float)x+1;
		   grid.p[1].y=(float)z;
		   grid.val[1]=pMapData->Voxel[x+1][y][z];

		   grid.p[2].x=(float)x+1;
		   grid.p[2].y=(float)z+1;
		   grid.val[2]=pMapData->Voxel[x+1][y][z+1];

		   grid.p[3].x=(float)x;
		   grid.p[3].y=(float)z+1;
		   grid.val[3]=pMapData->Voxel[x][y][z+1];
				
		   for (i=0;i<=2;i++)
		   {
		    if ((pM->iNcontourlines[i]+2)<MAXCONTOURLINES) // check the max avaaliable number of lines
			{
			 iNewLines= Polygonise2D(grid,double(pMapData->layer_limits[i]),&newlines[0]);
			 // copy lines from the buffer to destination
			 for (j=0;j<iNewLines;j++)
			 {
			  for(k=0;k<=1;k++) // copy the begining and end of the line to destination
			  {
				pM->contourlines[i][pM->iNcontourlines[i]+j].p[k]=
				IndToCoord3Df(newlines[j].p[k].x,(float)y,newlines[j].p[k].y,pMapData);
			  }// next k
			 }// next j
			 pM->iNcontourlines[i]=pM->iNcontourlines[i]+iNewLines;
			}// endif
			else
			{
				AfxMessageBox("g_iNcontourlines[i]+2)>MAXCONTOURLINES !!");
			}
		   }// next i							
		  }// next y
		 }// next x
		}// next z
		}// end case
	break;

	case Z: // map in Z direction, x,y of the 2D grid coresponds to 3D x,y
		{ 
		 for (z=0;z<zmax;z++)
		 {
	      for (x=0;x<(xmax-1);x++)
		  {
	       for (y=0;y<(ymax-1);y++)
		   { // prepare the grid values		
		   grid.p[0].x=(float)x;
		   grid.p[0].y=(float)y;
		   grid.val[0]=pMapData->Voxel[x][y][z];

		   grid.p[1].x=(float)x+1;
		   grid.p[1].y=(float)y;
		   grid.val[1]=pMapData->Voxel[x+1][y][z];

		   grid.p[2].x=(float)x+1;
		   grid.p[2].y=(float)y+1;
		   grid.val[2]=pMapData->Voxel[x+1][y+1][z];

		   grid.p[3].x=(float)x;
		   grid.p[3].y=(float)y+1;
		   grid.val[3]=pMapData->Voxel[x][y+1][z];
				
		   for (i=0;i<=2;i++)
		   {
		    if ((pM->iNcontourlines[i]+2)<MAXCONTOURLINES) // check the max avaaliable number of lines
			{
			 iNewLines= Polygonise2D(grid,double(pMapData->layer_limits[i]),&newlines[0]);
			 // copy lines from the buffer to destination
			 for (j=0;j<iNewLines;j++)
			 {
			  for(k=0;k<=1;k++) // copy the begining and end of the line to destination
			  {
				pM->contourlines[i][pM->iNcontourlines[i]+j].p[k]=
				IndToCoord3Df(newlines[j].p[k].x,newlines[j].p[k].y,(float)z,pMapData);
			  }// next k
			 }// next j
			 pM->iNcontourlines[i]=pM->iNcontourlines[i]+iNewLines;
			}// endif
			else
			{
				AfxMessageBox("m_iNcontourlines[i]+2)>MAXCONTOURLINES !!");
			}
		   }// next i							
		  }// next y
		 }// next x
		}// next z
		}// end case
	break;
	}

}

// 2D map version of the function above

void CMCEDataEngine::VoxelDirectionalContour2D(ELD_MapData *pMapData,contour_direction direction,LineMesh *pM)
{
	int x;
	int y;
	int z;
	int xmax;
	int ymax;
	int zmax;
	int i,j,k;

	int iNewLines;
	GRIDCELL2D grid;
	LINE2D newlines[2];

	xmax=pMapData->Lines; //X
	ymax=pMapData->Columns; //Y
	zmax=pMapData->Sections; //Z
	
	switch(direction)
	{

	case Z: // map in Z direction, x,y of the 2D grid coresponds to 3D x,y
		{ 
		 //for (z=0;z<zmax;z++)
		 z=pMapData->map_2D_level;
		 {
	      for (x=0;x<(xmax-1);x++)
		  {
	       for (y=0;y<(ymax-1);y++)
		   { // prepare the grid values		
		   grid.p[0].x=(float)x;
		   grid.p[0].y=(float)y;
		   grid.val[0]=pMapData->Voxel[x][y][z];

		   grid.p[1].x=(float)x+1;
		   grid.p[1].y=(float)y;
		   grid.val[1]=pMapData->Voxel[x+1][y][z];

		   grid.p[2].x=(float)x+1;
		   grid.p[2].y=(float)y+1;
		   grid.val[2]=pMapData->Voxel[x+1][y+1][z];

		   grid.p[3].x=(float)x;
		   grid.p[3].y=(float)y+1;
		   grid.val[3]=pMapData->Voxel[x][y+1][z];
				
		   for (i=0;i<=2;i++)
		   {
		    if ((pM->iNcontourlines[i]+2)<MAXCONTOURLINES) // check the max avaaliable number of lines
			{
			 iNewLines= Polygonise2D(grid,double(pMapData->layer_limits[i]),&newlines[0]);
			 // copy lines from the buffer to destination
			 for (j=0;j<iNewLines;j++)
			 {
			  for(k=0;k<=1;k++) // copy the begining and end of the line to destination
			  {
				pM->contourlines[i][pM->iNcontourlines[i]+j].p[k]=
				IndToCoord3Df(newlines[j].p[k].x,newlines[j].p[k].y,(float)z,pMapData);
			  }// next k
			 }// next j
			 pM->iNcontourlines[i]=pM->iNcontourlines[i]+iNewLines;
			}// endif
			else
			{
				AfxMessageBox("m_iNcontourlines[i]+2)>MAXCONTOURLINES !!");
			}
		   }// next i							
		  }// next y
		 }// next x
		}// next z
		}// end case
	break;

	}

}

int CMCEDataEngine::Polygonise2D(GRIDCELL2D grid, double isolevel, LINE2D *linefield)
{
   int nlines;
   int squareindex;
   int square_top[2][2];
   int i,j;

   /*edge	numbering scheme
'                 2
'             3------2
'             |      |
'            3|      |1
'             |      |
'             0------1
'                 0
	*/

/*
   XYZ vertlist[12];



   /*
      Determine the index into the edge table which
      tells us which vertices are inside of the square
   */

   squareindex = 0;
   if (grid.val[0] < isolevel) squareindex |= 1;
   if (grid.val[1] < isolevel) squareindex |= 2;
   if (grid.val[2] < isolevel) squareindex |= 4;
   if (grid.val[3] < isolevel) squareindex |= 8;
   
   /* Square is entirely in/out of the surface */

   if ((squareindex == 0) | (squareindex== 15))
      return(0);

   switch (squareindex)
   {
	case 1:
		nlines= 1;
		square_top[0][0] = 0;
		square_top[0][1] = 3;
	break;

	case  2:
		nlines = 1;
		square_top[0][0] = 0;
		square_top[0][1] = 1;
	break;
	
	case 3:
		nlines = 1;
		square_top[0][0] = 3;
		square_top[0][1] = 1;
	break;
	
	case 4:
		nlines = 1;
		square_top[0][0] = 1;
		square_top[0][1] = 2;
	break;
  
	case 5:
		nlines = 2;
		square_top[0][0] = 0;
		square_top[0][1] = 3;
		square_top[1][0] = 1;
		square_top[1][1] = 2;
	break;

	case 6:
		nlines = 1;
		square_top[0][0] = 0;
		square_top[0][1] = 2;
	break;
	
	case  7:
		nlines = 1;
		square_top[0][0] = 3;
		square_top[0][1] = 2;
	break;
  
	case 8:
		nlines = 1;
		square_top[0][0] = 3;
		square_top[0][1] = 2;
	break;

	case 9:
		nlines = 1;
		square_top[0][0] = 0;
		square_top[0][1] = 2;
	break;

	case 10:
		nlines = 2;
		square_top[0][0] = 0;
		square_top[0][1] = 1;
		square_top[1][0] = 2;
		square_top[1][1] = 3;
	break;

	case 11:
		nlines = 1;
		square_top[0][0] = 1;
		square_top[0][1] = 2;
	break;

	case 12:
		nlines = 1;
		square_top[0][0] = 1;
		square_top[0][1] = 3;
	break;

	case 13:
		nlines = 1;
		square_top[0][0] = 0;
		square_top[0][1] = 1;
	break;

	case 14:
		nlines = 1;
		square_top[0][0] = 0;
		square_top[0][1] = 3;
	break;
   }

   XY p;
   // debuging
   p.x=0;
   p.y=0;


   for (i=0;i<nlines;i++) // i mut be in <0,1> range !
   {
	 for(j=0;j<=1;j++) // first and second line point
	 {
	   switch (square_top[i][j])
	   {
		   case 0:
				linefield[i].p[j]=VertexInterp2D(isolevel,grid.p[0],grid.p[1],grid.val[0],grid.val[1]);
		   break;

		   case 1:
   				linefield[i].p[j]=VertexInterp2D(isolevel,grid.p[1],grid.p[2],grid.val[1],grid.val[2]);
		   break;

		   case 2:
			 	linefield[i].p[j]=VertexInterp2D(isolevel,grid.p[2],grid.p[3],grid.val[2],grid.val[3]);
		   break;

		   case 3:
			 	linefield[i].p[j]=VertexInterp2D(isolevel,grid.p[3],grid.p[0],grid.val[3],grid.val[0]);
		   break;

		}
	 }			
   }

	return nlines;
}

#define ABS(x) (x < 0 ? -(x) : (x))

XY CMCEDataEngine::VertexInterp2D(double isolevel, XY p1, XY p2, double valp1, double valp2)
{
   double mu;
   XY p;

   if (ABS(isolevel-valp1) < 0.00001)
      return(p1);
   if (ABS(isolevel-valp2) < 0.00001)
      return(p2);
   if (ABS(valp1-valp2) < 0.00001)
      return(p1);

   mu = (isolevel - valp1) / (valp2 - valp1);
   p.x =float( p1.x + mu * (p2.x - p1.x));
   p.y =float( p1.y + mu * (p2.y - p1.y));

   return p;
}

void CMCEDataEngine::GenerateDisplayLists()
{
	unsigned long i,j,k;

	// 3D map line controur generation
	glNewList(MCE_CONTOURS, GL_COMPILE);
	for (j=0;j<NMAPS;j++) // maps cycling
	{
		for (k=0;k<3;k++) // layer cycling
		{
			glColor3f (M[j].layer_color[k].R,
					   M[j].layer_color[k].G,
					   M[j].layer_color[k].B); // layer specific collor ...

			if(MolData[j].MapData.layer_visibility[k])
			{
				for (i=0;i<M[j].iNcontourlines[k];i++) // individual line loop
				{
				glBegin(GL_LINE_STRIP);
					glVertex3f(M[j].contourlines[k][i].p[0].x,
							   M[j].contourlines[k][i].p[0].y,
							   M[j].contourlines[k][i].p[0].z);
					glVertex3f(M[j].contourlines[k][i].p[1].x,
							   M[j].contourlines[k][i].p[1].y,
							   M[j].contourlines[k][i].p[1].z);
				glEnd();
				}
			}
		}
	}
	glEndList();

	// 2D map line controur generation
	glNewList(MCE_CONTOURS2D, GL_COMPILE);
	for (j=0;j<NMAPS;j++) // maps cycling
	{
		// we have ony yes/now for whole 2D map ....
		if(MolData[j].MapData.map_2D_visibility)
		{
			for (k=0;k<3;k++) // layer cycling
			{
				// we will use for 2D map identical color as for 3D map 
				// for simplicity ...
				glColor3f (M[j].layer_color[k].R,
						   M[j].layer_color[k].G,
			
						   M[j].layer_color[k].B); // layer specific collor ...

				for (i=0;i<M2D[j].iNcontourlines[k];i++) // individual line loop
				{
				glBegin(GL_LINE_STRIP);
					glVertex3f(M2D[j].contourlines[k][i].p[0].x,
							   M2D[j].contourlines[k][i].p[0].y,
							   M2D[j].contourlines[k][i].p[0].z);
					glVertex3f(M2D[j].contourlines[k][i].p[1].x,
							   M2D[j].contourlines[k][i].p[1].y,
							   M2D[j].contourlines[k][i].p[1].z);
				glEnd();
				}
			}
			// draw the map borders
			// points at the rectangle ends ...
			XYZ p0=CMCEDataEngine::IndToCoord3Df(0,0,MolData[j].MapData.map_2D_level, &MolData[j].MapData);
			XYZ p1=CMCEDataEngine::IndToCoord3Df(MolData[j].MapData.Lines,0,MolData[j].MapData.map_2D_level, &MolData[j].MapData);
			XYZ p2=CMCEDataEngine::IndToCoord3Df(MolData[j].MapData.Lines,MolData[j].MapData.Columns,MolData[j].MapData.map_2D_level, &MolData[j].MapData);
			XYZ p3=CMCEDataEngine::IndToCoord3Df(0,MolData[j].MapData.Columns,MolData[j].MapData.map_2D_level, &MolData[j].MapData);

			glColor3f(0.7,0.7,0.7);
			glLineWidth(2); // temporaly change

			glBegin(GL_LINE_LOOP);
				glVertex3f (p0.x,p0.y,p0.z);
				glVertex3f (p1.x,p1.y,p1.z);
				glVertex3f (p2.x,p2.y,p2.z);
				glVertex3f (p3.x,p3.y,p3.z);
			glEnd();

			glLineWidth(1); // set back line width value
		}
	}
	glEndList();

}

void CMCEDataEngine::StuctureToCart(MoleculeData *atoms, ELD_MapData *map)
{
int i;

	for (i=0;i<atoms->Natoms;i++)
	{
		atoms->a[i].cc=CalcFractToCart(atoms->a[i].fc,map->PlaneTranslation,map->PlaneMatrix);
	}
}

XYZ CMCEDataEngine::CalcFractToCart(XYZ fc, float translation[], float matrix[][3])
{
XYZ result;
float translated[3];
float rotated[3];

	translated[0]=fc.x-translation[0];
	translated[1]=fc.y-translation[1];
	translated[2]=fc.z-translation[2];

	MultBy33Matrix(translated,rotated,matrix);

	result.x=rotated[0];
	result.y=rotated[1];
	result.z=rotated[2];

return result;
}

XYZ CMCEDataEngine::CalcCartToFract(XYZ cc, float translation[], float matrix[][3])
{
XYZ fc;
float original[3];
float transformed[3];
float imatrix[3][3];

	original[0]=cc.x;
	original[1]=cc.y;
	original[2]=cc.z;

	InvertMatrix(matrix,imatrix);

	MultBy33Matrix(original,transformed,imatrix);

	fc.x=transformed[0]+translation[0];
	fc.y=transformed[1]+translation[1];
	fc.z=transformed[2]+translation[2];

return fc;
}



void CMCEDataEngine::MultBy33Matrix(float v1[], float v2[], float matrix[][3])
{
//v1 input vector
//v2 result
//Matrix 3x3 after multiplication
int i;
int j;

	for (i=0;i<=2;i++)
	{
		v2[i] = 0;
		for(j=0;j<=2;j++)
		{
			v2[i] = v2[i] + matrix[i][j] * v1[j];
		} //next j
	}// next i
}

void CMCEDataEngine::SetupConectivity(MoleculeData *atoms)
{
 int i, j;
 float blenght;
 float LimitMin, LimitMax;
 float COVR1,COVR2,Tolerance1,Tolerance2;

 // transfer atom names to periodic table position numbers ..
 DecodeAtomNames(atoms);

 atoms->bond.nBond=0;

 for(i=0; i<atoms->Natoms; i++)
 {
	 for(j=i+1; j<atoms->Natoms;j++)
	 {
		 blenght=distance(atoms->a[i].cc,atoms->a[j].cc);

		// set the bond interval for given atom combination ...
		COVR1=m_ElementsProperties[atoms->a[i].AtomNumber].COVR;
		COVR2=m_ElementsProperties[atoms->a[j].AtomNumber].COVR;
		Tolerance1=m_ElementsProperties[atoms->a[i].AtomNumber].Tolerance;
		Tolerance2=m_ElementsProperties[atoms->a[j].AtomNumber].Tolerance;
	    LimitMin = (COVR1 + COVR2) - (Tolerance1 + Tolerance2);
        LimitMax = (COVR1 + COVR2) + (Tolerance1 + Tolerance2);

		 // shuld be replaced by property table int the future
	
		if((blenght>LimitMin)&&(blenght<LimitMax)) // new bond
		{
			atoms->bond.nIndex[atoms->bond.nBond][0]=i;
			atoms->bond.nIndex[atoms->bond.nBond][1]=j;
			atoms->bond.nBond=atoms->bond.nBond++;
		}

	 } // next j
 }

}

float CMCEDataEngine::distance(XYZ p1, XYZ p2)
{
// simple distance of two points
float d1,d2,d3;

	d1=p1.x-p2.x;
	d2=p1.y-p2.y;
	d3=p1.z-p2.z;

return (float)sqrt(d1*d1+d2*d2+d3*d3);
}

void CMCEDataEngine::DrawAtoms()
{
	int i;

	static GLUquadricObj *quadObj3=NULL;

	// sphere matereial
	GLfloat mat_specular[]={1.0,1.0,1.0,1.0};
	GLfloat mat_shiness[]={50.0};

	glMaterialfv(GL_FRONT,GL_SPECULAR,mat_specular);
	glMaterialfv(GL_FRONT,GL_SHININESS,mat_shiness);
	
  // sphere 
	if (! quadObj3)
	{
		quadObj3 = gluNewQuadric();
		//object properties (invisible if not defined !)
		//type of filling LINE,FILL
		gluQuadricDrawStyle (quadObj3, GLU_FILL);
		//type of shading - SMOOTH,FLAT
		gluQuadricNormals (quadObj3, GLU_SMOOTH);
		//vektors INSIDE OUTSIDE - for light calculations ??
		gluQuadricOrientation (quadObj3, GLU_OUTSIDE);
	}
	
	for(i=0;i<MolData[0].Molecule.Natoms;i++)
	{
	glPushMatrix();

		SetAtomColor(MolData[0].Molecule.a[i].name);

		glTranslatef(MolData[0].Molecule.a[i].cc.x,
					 MolData[0].Molecule.a[i].cc.y,
				     MolData[0].Molecule.a[i].cc.z);

		// shuld be replaced by radius table in the future ...
		
		if (strncmp(MolData[0].Molecule.a[i].name,"H ",2)==0)
		{
			gluSphere (quadObj3, 0.1 ,8,8);
		}
		else
		{
			gluSphere (quadObj3, 0.2 ,8,8);
		}

	glPopMatrix();
	}
}

void CMCEDataEngine::SetAtomColor(char name[])
{
	if (strncmp(name,"C ",2)==0)
	{
		glColor3f(0.7,0.7,0.7); // gray
	}

	else if (strncmp(name,"H ",2)==0)
	{
		glColor3f(0.7,0.7,1.0); // light blue
	}

	else if (strncmp(name,"N ",2)==0)
	{
		glColor3f(0,0,0.8); // blue
	}

	else if (strncmp(name,"O ",2)==0)
	{
		glColor3f(0.8,0,0); // red
	}

	else if (strncmp(name,"S ",2)==0)
	{
		glColor3f(0.8,0.8,0); // yellow
	}

	else
	{
		glColor3f(0.4,0.4,0.4); // dark gray
	}
}

void CMCEDataEngine::DrawBonds()
{
	int i;
	int i1;
	int i2;

	glColor3f(0.8,.8,.8); // gray lines for bonds

	glBegin(GL_LINES);

	for(i=0;i<MolData[0].Molecule.bond.nBond;i++)
	{
		i1=MolData[0].Molecule.bond.nIndex[i][0];
		i2=MolData[0].Molecule.bond.nIndex[i][1];

		glVertex3f(MolData[0].Molecule.a[i1].cc.x,
				  MolData[0].Molecule.a[i1].cc.y,
				  MolData[0].Molecule.a[i1].cc.z);

		glVertex3f(MolData[0].Molecule.a[i2].cc.x,
			       MolData[0].Molecule.a[i2].cc.y,
				   MolData[0].Molecule.a[i2].cc.z);
	}

	glEnd();
}

XYZ CMCEDataEngine::IndToCoord3Df(float x, float y, float z, ELD_MapData *map)
{
XYZ result;

	result.x=(map->AxisParam[0].Start + x * map->AxisParam[0].Step);

	result.y=(map->AxisParam[1].Start + y * map->AxisParam[1].Step);

	result.z=(map->AxisParam[2].Start + z * map->AxisParam[2].Step);

return result;
}

BOOL CMCEDataEngine::LoadInitialValues()
{
	// setup of inital values ...

	MarkPosition.x=0;
	MarkPosition.y=0;
	MarkPosition.z=0;

	// map satate
	MolData[0].bInitialized=FALSE;
	MolData[1].bInitialized=FALSE;

	// set the initial values to peak list
	PeakList.Natoms=0;

	//
	bMarkVisible=FALSE;

	//  laer color values setup ...
	M[0].layer_color[0].R=0; // green
	M[0].layer_color[0].G=1;
	M[0].layer_color[0].B=0;

	M[0].layer_color[1].R=0; // blue
	M[0].layer_color[1].G=0;
	M[0].layer_color[1].B=1;

	M[0].layer_color[2].R=1; // red
	M[0].layer_color[2].G=0;
	M[0].layer_color[2].B=0;

	M[1].layer_color[0].R=1; // yelow
	M[1].layer_color[0].G=1;
	M[1].layer_color[0].B=0;

	M[1].layer_color[1].R=0;   // sky blue
	M[1].layer_color[1].G=1; 
	M[1].layer_color[1].B=1;

	M[1].layer_color[2].R=1; // purple
	M[1].layer_color[2].G=0;
	M[1].layer_color[2].B=1;

	// Setup default parameters

	Setup.bUnitCellVisible = TRUE;
	Setup.bMoleculeVisible = TRUE;
	Setup.PickMode = Picking3D;
//	Setup.PickMode = Picking2D;
	Setup.PerspMode = Perspective;
//	Setup.PerspMode = Orthographic;


	// find the application current loaction and load the setups from this location ....
	char szBuffer[_MAX_PATH]; 

	VERIFY(::GetModuleFileName(AfxGetInstanceHandle(), szBuffer, _MAX_PATH));

	CString sPath = (CString)szBuffer;
	sPath = sPath.Left(sPath.ReverseFind('\\'));	

    sPath=sPath+"\\atompar2.txt";

	CFile f(sPath, CFile::modeRead);

	CArchive ar(&f,CArchive::load);

	m_nElementsNumber=0;

	CString strInputData;

	ar.ReadString(strInputData); //Atom  COVR   GRAR   VDWR   T   ATSYMB   ATWE

	do
	{
		ar.ReadString(strInputData);

		if(atof(strInputData.Mid(1,3))<0) break;

		m_nElementsNumber++; // warning we will number thie filed from 1 insted from 0 !!!

		m_ElementsProperties[m_nElementsNumber].AtomNumber=atoi(strInputData.Mid(1,3));
		m_ElementsProperties[m_nElementsNumber].COVR=atof(strInputData.Mid(6,3))/100;
		m_ElementsProperties[m_nElementsNumber].GRAR=atof(strInputData.Mid(13,3))/100;
		m_ElementsProperties[m_nElementsNumber].VDWR=atof(strInputData.Mid(20,3))/100;
		m_ElementsProperties[m_nElementsNumber].Tolerance=atof(strInputData.Mid(27,3));
		strcpy(m_ElementsProperties[m_nElementsNumber].name,strInputData.Mid(33,2));
		m_ElementsProperties[m_nElementsNumber].AtomWeight=atof(strInputData.Mid(37,10));

	} while(TRUE);

	ar.Close();
	f.Close();

	// TODO load other defalt configuration .....

return TRUE;
}

void CMCEDataEngine::DecodeAtomNames(MoleculeData *atoms)
{
/*
Dim i As Integer
Dim j As Integer
For i = 1 To PocetAtomu
    j = 0
    Do
      j = j + 1
      ' identify the atom name, both strings must be 4 chracter lenght
      If AtomNames(i) = AtomProperties(j, 6) Then
         AtomDescription(i, 13) = AtomProperties(j, 1)
         Exit Do
      End If
      If j > AtomProperties(0, 1) Then
        MsgBox "I do not know this atom name: " + AtomNames(i)
        Stop
      End If
    Loop
Next i
*/

int i, j;
for(i=0; i<atoms->Natoms; i++)
{
	for(j=1;j<=m_nElementsNumber; j++)
	{
		if(strncmp(atoms->a[i].name,m_ElementsProperties[j].name,2)==0)
		{
			atoms->a[i].AtomNumber=m_ElementsProperties[j].AtomNumber;
			break;
		}
	}
	if (j>m_nElementsNumber)
	{
		AfxMessageBox("Unknown element name in your data");
	}
}

}

void CMCEDataEngine::DrawMark()
{
// draw sphere at position 0,0,0 for beggining ....
	static GLUquadricObj *quadObj1=NULL;
	const 	float size=0.6f;

	// sphere matereial
	GLfloat mat_specular[]={1.0,1.0,1.0,1.0};
	GLfloat mat_shiness[]={50.0};

	glMaterialfv(GL_FRONT,GL_SPECULAR,mat_specular);
	glMaterialfv(GL_FRONT,GL_SHININESS,mat_shiness);
	
  // sphere 
	if (! quadObj1)
	{
		quadObj1 = gluNewQuadric();
		//object properties (invisible if not defined !)
		//type of filling LINE,FILL
		gluQuadricDrawStyle (quadObj1, GLU_FILL);
		//type of shading - SMOOTH,FLAT
		gluQuadricNormals (quadObj1, GLU_SMOOTH);
		//vektors INSIDE OUTSIDE - for light calculations ??
		gluQuadricOrientation (quadObj1, GLU_OUTSIDE);
	}

	if(bMarkVisible)
	{

	// mark point
	glDisable(GL_LIGHTING);

		glBegin(GL_LINES);
			glColor3f(1,1,1); // X
			glVertex3f(MarkPosition.x-size,MarkPosition.y,MarkPosition.z);
			glVertex3f(MarkPosition.x+size,MarkPosition.y,MarkPosition.z);
		glEnd();

		glBegin(GL_LINES);
			glColor3f(1,1,1); // Y
			glVertex3f(MarkPosition.x,MarkPosition.y-size,MarkPosition.z);
			glVertex3f(MarkPosition.x,MarkPosition.y+size,MarkPosition.z);
		glEnd();
		glBegin(GL_LINES);
			glColor3f(1,1,1); // Z
			glVertex3f(MarkPosition.x,MarkPosition.y,MarkPosition.z+size);
			glVertex3f(MarkPosition.x,MarkPosition.y,MarkPosition.z-size);
		glEnd();		

	glEnable(GL_LIGHTING);

	glPushMatrix();

		glColor3f (0.9,0.7,0.2); //orange

		glTranslatef(MarkPosition.x,
					 MarkPosition.y,
					 MarkPosition.z);
		
		gluSphere (quadObj1, 0.22 ,8,8);

	glPopMatrix();
	}
}

float CMCEDataEngine::Intensity(float x, float y, float z)
{
	float intensity;
	int ix,iy,iz;
	int xmax,ymax,zmax;
	XYZ voxel_pos;

	xmax=MolData[0].MapData.Lines;
	ymax=MolData[0].MapData.Columns;
	zmax=MolData[0].MapData.Sections;


	// totaly simple algorithm finding the
	// voxel nearest to the position ve examine ...

	voxel_pos=Coord3DfToInd(x,y,z,&MolData[0].MapData);
	
	ix=(int)(voxel_pos.x+0.5);
	iy=(int)(voxel_pos.y+0.5);
	iz=(int)(voxel_pos.z+0.5);
	

	if ((ix>=0) & (iy>=0) & (iz >=0) & (ix<=xmax) & (iy<=ymax) & (iz<=zmax))
	{
		intensity=MolData[0].MapData.Voxel[ix][iy][iz];
	}
	else
	{
		intensity=-10000;
	}

	return intensity;
}

XYZ CMCEDataEngine::Coord3DfToInd(float x, float y, float z, ELD_MapData *MapData)
{
XYZ result;

	result.x=(x-MapData->AxisParam[0].Start)/MapData->AxisParam[0].Step;

	result.y=(y-MapData->AxisParam[1].Start)/MapData->AxisParam[1].Step;

	result.z=(z-MapData->AxisParam[2].Start)/MapData->AxisParam[2].Step;

return result;
}

void CMCEDataEngine::DrawPeaks()
{
	int i;

	static GLUquadricObj *quadObj2=NULL;

	// sphere matereial
	GLfloat mat_specular[]={1.0,1.0,1.0,1.0};
	GLfloat mat_shiness[]={50.0};

	glMaterialfv(GL_FRONT,GL_SPECULAR,mat_specular);
	glMaterialfv(GL_FRONT,GL_SHININESS,mat_shiness);
	
  // sphere 
	if (! quadObj2)
	{
		quadObj2 = gluNewQuadric();
		//object properties (invisible if not defined !)
		//type of filling LINE,FILL
		gluQuadricDrawStyle (quadObj2, GLU_FILL);
		//type of shading - SMOOTH,FLAT
		gluQuadricNormals (quadObj2, GLU_SMOOTH);
		//vektors INSIDE OUTSIDE - for light calculations ??
		gluQuadricOrientation (quadObj2, GLU_OUTSIDE);
	}
	
	for(i=0;i<PeakList.Natoms;i++)
	{
	glPushMatrix();

		glColor3f(1,1,0) ; //yellow  ...

		glTranslatef(PeakList.a[i].cc.x,
					 PeakList.a[i].cc.y,
				     PeakList.a[i].cc.z);

		gluSphere (quadObj2, 0.20 ,8,8);

	glPopMatrix();
	}
}

void CMCEDataEngine::Addnewpeak()
{
	PeakList.a[PeakList.Natoms].cc=MarkPosition;

	PeakList.Natoms++;
}

void CMCEDataEngine::SavePeaks()
{
	CFileDialog dlg(FALSE, "dat", "newatoms.dat", OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "CRYSTALS command file (*.dat)|*.dat|All Files (*.*)|*.*| |");

	char szBuffer[250];
	int i;
		
	if(dlg.DoModal() == IDOK)
	{
		CFile f( dlg.GetPathName(), CFile::modeCreate | CFile::modeWrite);
		CArchive ar(&f,CArchive::store);

		// calculate fc from cc
		for (i=0;i<PeakList.Natoms;i++)
		{
			PeakList.a[i].fc=CalcCartToFract(PeakList.a[i].cc,
			MolData[0].MapData.PlaneTranslation,
			MolData[0].MapData.PlaneMatrix);
		}
	
		ar.WriteString("#EDIT\n"); //CRYSTAL command

		for (i=0;i<PeakList.Natoms;i++)
		{
			// CRYSTALS formated atom list
			sprintf(szBuffer,"ATOM Q  1   X=%10.6f  Y=%10.6f  Z=%10.6f \n",
			PeakList.a[i].fc.x,PeakList.a[i].fc.y,PeakList.a[i].fc.z);
			ar.WriteString(szBuffer);
		}

		ar.WriteString("END\n"); //CRYSTAL command

		ar.Close();
		f.Close();
	}
}

void CMCEDataEngine::SavePovRayInput()
{
	CFileDialog dlg(FALSE, "pov", "MCE.pov", OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "POV-ray input file (*.pov)|*.pov|All Files (*.*)|*.*| |");

	char szBuffer[250];
	int i;
		
	if(dlg.DoModal() == IDOK)
	{
		CFile f( dlg.GetPathName(), CFile::modeCreate | CFile::modeWrite);
		CArchive ar(&f,CArchive::store);

		// make a copy of the POV header file to the destination ...

		VERIFY(::GetModuleFileName(AfxGetInstanceHandle(), szBuffer, _MAX_PATH));

		CString sPath = (CString)szBuffer;
		sPath = sPath.Left(sPath.ReverseFind('\\'));	

		sPath=sPath+"\\MCE_POVray_head.def";

		CFile f_header(sPath, CFile::modeRead);
		
		CArchive ar_header(&f_header,CArchive::load);

		CString strInputData;

		do
		{
			ar_header.ReadString(strInputData);

			if(strInputData.Left(13)=="//END OF FILE") break;

			ar.WriteString(strInputData);
			ar.WriteString("\n");
		
		} while(TRUE);

		ar_header.Close();
		f_header.Close();

		// drawing of the scene
		ar.WriteString("union {\n");
		ar.WriteString("union {\n");
		
		//POVDrawMark();
		POVDrawPeaks(&ar);

		if(Setup.bMoleculeVisible)
		{
			POVDrawBonds(&ar);
			POVDrawAtoms(&ar);
		}

		if(Setup.bUnitCellVisible)
		{
			POVDrawUnitCell(&ar);
		}

		POVMceContours(&ar);

		POVMceContours2D(&ar);

		POVSaveMatrix(&ar);

		ar.WriteString("}\n");
		ar.WriteString("scale <1,1,-1>\n");
		ar.WriteString("}\n");

		ar.Close();
		f.Close();
	}
}

void CMCEDataEngine::InvertMatrix(float m[3][3], float mi[3][3])
{
float detM;   // Determinant
float D[3][3];  // Algeb. reminder
float DT[3][3]; // Transponed algeb. reminder
int i, j;

detM =  (m[0][0] * m[1][1] * m[2][2]
	   + m[1][0] * m[2][1] * m[0][2]
	   + m[2][0] * m[0][1] * m[1][2])
	   -(m[2][0] * m[1][1] * m[0][2]
	   + m[0][0] * m[2][1] * m[1][2]
	   + m[1][0] * m[0][1] * m[2][2]);

D[0][0] =  (m[1][1] * m[2][2] - m[2][1] * m[1][2]);
D[0][1] = -(m[1][0] * m[2][2] - m[2][0] * m[1][2]);
D[0][2] =  (m[1][0] * m[2][1] - m[2][0] * m[1][1]);
D[1][0] = -(m[0][1] * m[2][2] - m[2][1] * m[0][2]);
D[1][1] =  (m[0][0] * m[2][2] - m[2][0] * m[0][2]);
D[1][2] = -(m[0][0] * m[2][1] - m[2][0] * m[0][1]);
D[2][0] =  (m[0][1] * m[1][2] - m[1][1] * m[0][2]);
D[2][1] = -(m[0][0] * m[1][2] - m[1][0] * m[0][2]);
D[2][2] =  (m[0][0] * m[1][1] - m[1][0] * m[0][1]);

for(i = 0; i<3;i++)
{
    for(j=0; j<3;j++)
	{
		DT[i][j] = D[j][i];
	}
}

for(i = 0; i<3;i++)
{
    for(j=0; j<3;j++)
	{
		mi[i][j] = 1 / detM * DT[i][j];
	}
}

}

void CMCEDataEngine::DrawUnitCell()
{
	int i;

	glColor3f(0.7, 0.7, 0.7); //pure white is not suitable for hardcopy
	glLineWidth(2);

	glBegin (GL_LINE_LOOP);
    for(i=0;i<4;i++)
	{
        glVertex3f(UnitCellEndPoints[i].x,
				   UnitCellEndPoints[i].y,
				   UnitCellEndPoints[i].z);
	}
	glEnd();

	glBegin (GL_LINE_LOOP);
    for(i=4;i<8;i++)
	{
        glVertex3f(UnitCellEndPoints[i].x,
			       UnitCellEndPoints[i].y,
				   UnitCellEndPoints[i].z);
	}
	glEnd();
 
  glBegin (GL_LINES);
    glVertex3f(UnitCellEndPoints[0].x, UnitCellEndPoints[0].y, UnitCellEndPoints[0].z);
    glVertex3f(UnitCellEndPoints[4].x, UnitCellEndPoints[4].y, UnitCellEndPoints[4].z);
    glVertex3f(UnitCellEndPoints[1].x, UnitCellEndPoints[1].y, UnitCellEndPoints[1].z);
    glVertex3f(UnitCellEndPoints[5].x, UnitCellEndPoints[5].y, UnitCellEndPoints[5].z);
    glVertex3f(UnitCellEndPoints[2].x, UnitCellEndPoints[2].y, UnitCellEndPoints[2].z);
    glVertex3f(UnitCellEndPoints[6].x, UnitCellEndPoints[6].y, UnitCellEndPoints[6].z);
    glVertex3f(UnitCellEndPoints[3].x, UnitCellEndPoints[3].y, UnitCellEndPoints[3].z);
    glVertex3f(UnitCellEndPoints[7].x, UnitCellEndPoints[7].y, UnitCellEndPoints[7].z);
  glEnd();

  glLineWidth(1);

}

void CMCEDataEngine::GenerateLatticePoint(ELD_MapData *pMapData)
{
	XYZ e_p;

	e_p.x=0;e_p.y=0;e_p.z=0;
	UnitCellEndPoints[0]=CalcFractToCart(e_p,pMapData->PlaneTranslation,pMapData->PlaneMatrix);
	e_p.x=1;e_p.y=0;e_p.z=0;
	UnitCellEndPoints[1]=CalcFractToCart(e_p,pMapData->PlaneTranslation,pMapData->PlaneMatrix);
	e_p.x=1;e_p.y=1;e_p.z=0;
	UnitCellEndPoints[2]=CalcFractToCart(e_p,pMapData->PlaneTranslation,pMapData->PlaneMatrix);
	e_p.x=0;e_p.y=1;e_p.z=0;
	UnitCellEndPoints[3]=CalcFractToCart(e_p,pMapData->PlaneTranslation,pMapData->PlaneMatrix);
	e_p.x=0;e_p.y=0;e_p.z=1;
	UnitCellEndPoints[4]=CalcFractToCart(e_p,pMapData->PlaneTranslation,pMapData->PlaneMatrix);
	e_p.x=1;e_p.y=0;e_p.z=1;
	UnitCellEndPoints[5]=CalcFractToCart(e_p,pMapData->PlaneTranslation,pMapData->PlaneMatrix);
	e_p.x=1;e_p.y=1;e_p.z=1;
	UnitCellEndPoints[6]=CalcFractToCart(e_p,pMapData->PlaneTranslation,pMapData->PlaneMatrix);
	e_p.x=0;e_p.y=1;e_p.z=1;
	UnitCellEndPoints[7]=CalcFractToCart(e_p,pMapData->PlaneTranslation,pMapData->PlaneMatrix);
}


void CMCEDataEngine::POVSaveMatrix(CArchive* ar)
{
GLfloat matrix[16];

// glMatrixMode(GL_MODELVIEW);

// frmMain.recalcModelView

	char szBuffer[400];

	glGetFloatv (GL_MODELVIEW_MATRIX , matrix);
	
	ar->WriteString("matrix <\n");

	sprintf(szBuffer,"%f , %f , %f ,\n",matrix[0],matrix[1],matrix[2]);
	ar->WriteString(szBuffer);

	sprintf(szBuffer,"%f , %f , %f ,\n",matrix[4],matrix[5],matrix[6]);
	ar->WriteString(szBuffer);

	sprintf(szBuffer,"%f , %f , %f ,\n",matrix[8],matrix[9],matrix[10]);
	ar->WriteString(szBuffer);

	sprintf(szBuffer,"%f , %f , %f  \n",matrix[12],matrix[13],matrix[14]);
	ar->WriteString(szBuffer);

	ar->WriteString(">\n");

}

void CMCEDataEngine::POVDrawBonds(CArchive *ar)
{
	int i;
	int i1;
	int i2;

	ar->WriteString("union {\n");

	for(i=0;i<MolData[0].Molecule.bond.nBond;i++)
	{
		i1=MolData[0].Molecule.bond.nIndex[i][0];
		i2=MolData[0].Molecule.bond.nIndex[i][1];

		POVcylinder(MolData[0].Molecule.a[i1].cc,
					MolData[0].Molecule.a[i2].cc,
					"bondcylRadius",ar);
					
	}

	ar->WriteString("pigment {color Gray70}\n");
	ar->WriteString("}\n");
}

void CMCEDataEngine::POVcylinder(XYZ p1, XYZ p2, CString radius,CArchive *ar)
{
	char szBuffer[400];

	if(distance(p1,p2)>0.001) // eliminate degenerated cylindres ...
	{
		sprintf(szBuffer,"cylinder{< %f , %f , %f > , ",p1.x,p1.y,p1.z);
		ar->WriteString(szBuffer);

		sprintf(szBuffer,"< %f , %f , %f > , ",p2.x,p2.y,p2.z);
		ar->WriteString(szBuffer);

		sprintf(szBuffer,"%s}\n",radius);
		ar->WriteString(szBuffer);
	}
}

void CMCEDataEngine::POVDrawAtoms(CArchive *ar)
{
	int i;
	char szBuffer[400];
	char szAtomName[3];
 
	for(i=0;i<MolData[0].Molecule.Natoms;i++)
	{
		sprintf(szBuffer,"sphere {< %f , %f , %f > , ",
							MolData[0].Molecule.a[i].cc.x,
							MolData[0].Molecule.a[i].cc.y,
							MolData[0].Molecule.a[i].cc.z);
		ar->WriteString(szBuffer);

		// extract the atom name 
		sprintf(szAtomName,""); // clear the string
		strncat(szAtomName,MolData[0].Molecule.a[i].name,2);

		sprintf(szBuffer,"AtomRadiusFor%s",szAtomName);
		ar->WriteString(szBuffer);

		sprintf(szBuffer," pigment {color ColorFor%s}}\n",szAtomName);
		ar->WriteString(szBuffer);
	}	   

}

void CMCEDataEngine::POVMceContours(CArchive *ar)
{
	unsigned long i,j,k;
	char szBuffer[400];

	for (j=0;j<NMAPS;j++) // maps cycling
	{
		for (k=0;k<3;k++) // layer cycling
		{
			if(MolData[j].MapData.layer_visibility[k])
			{
				ar->WriteString("union {\n");

				for (i=0;i<M[j].iNcontourlines[k];i++) // individual line loop
				{
					POVcylinder	(M[j].contourlines[k][i].p[0],
								 M[j].contourlines[k][i].p[1],
								 "cylRadius",ar);
				}

				// color selection
				sprintf(szBuffer,"pigment { color Map%dLevel%dColor}}\n",j,k);				
				ar->WriteString(szBuffer);
			}
		}
	}
}

void CMCEDataEngine::POVMceContours2D(CArchive *ar)
{
	unsigned long i,j,k;
	char szBuffer[400];

	for (j=0;j<NMAPS;j++) // maps cycling
	{
		// we have ony yes/now for whole 2D map ....
		if(MolData[j].MapData.map_2D_visibility)
		{
			for (k=0;k<3;k++) // layer cycling
			{
				ar->WriteString("union {\n");

				for (i=0;i<M2D[j].iNcontourlines[k];i++) // individual line loop
				{
					POVcylinder(M2D[j].contourlines[k][i].p[0],
								M2D[j].contourlines[k][i].p[1],
								"cylRadius",ar);

				}

				// color selection
				sprintf(szBuffer,"pigment { color Map%dLevel%dColor } }\n",j,k);				
				ar->WriteString(szBuffer);

			}

			// draw the map borders
			// points at the rectangle ends ...
			XYZ p0=CMCEDataEngine::IndToCoord3Df(0,0,MolData[j].MapData.map_2D_level, &MolData[j].MapData);
			XYZ p1=CMCEDataEngine::IndToCoord3Df(MolData[j].MapData.Lines,0,MolData[j].MapData.map_2D_level, &MolData[j].MapData);
			XYZ p2=CMCEDataEngine::IndToCoord3Df(MolData[j].MapData.Lines,MolData[j].MapData.Columns,MolData[j].MapData.map_2D_level, &MolData[j].MapData);
			XYZ p3=CMCEDataEngine::IndToCoord3Df(0,MolData[j].MapData.Columns,MolData[j].MapData.map_2D_level, &MolData[j].MapData);

			ar->WriteString("union {\n");
			POVcylinder(p0,p1,"drawRadius",ar);
			POVcylinder(p1,p2,"drawRadius",ar);
			POVcylinder(p2,p3,"drawRadius",ar);
			POVcylinder(p3,p0,"drawRadius",ar);

			ar->WriteString("pigment {color Gray80}}\n");		
		}
	}
}

void CMCEDataEngine::POVDrawUnitCell(CArchive *ar)
{
	ar->WriteString("union {\n");

	POVcylinder(UnitCellEndPoints[0],UnitCellEndPoints[1],"drawRadius",ar);
	POVcylinder(UnitCellEndPoints[1],UnitCellEndPoints[2],"drawRadius",ar);
	POVcylinder(UnitCellEndPoints[2],UnitCellEndPoints[3],"drawRadius",ar);
	POVcylinder(UnitCellEndPoints[3],UnitCellEndPoints[0],"drawRadius",ar);

	POVcylinder(UnitCellEndPoints[4],UnitCellEndPoints[5],"drawRadius",ar);
	POVcylinder(UnitCellEndPoints[5],UnitCellEndPoints[6],"drawRadius",ar);
	POVcylinder(UnitCellEndPoints[6],UnitCellEndPoints[7],"drawRadius",ar);
	POVcylinder(UnitCellEndPoints[7],UnitCellEndPoints[4],"drawRadius",ar);

	POVcylinder(UnitCellEndPoints[0],UnitCellEndPoints[4],"drawRadius",ar);
	POVcylinder(UnitCellEndPoints[1],UnitCellEndPoints[5],"drawRadius",ar);
	POVcylinder(UnitCellEndPoints[2],UnitCellEndPoints[6],"drawRadius",ar);
	POVcylinder(UnitCellEndPoints[3],UnitCellEndPoints[7],"drawRadius",ar);

	ar->WriteString("pigment {color Gray80}}\n");		
}

void CMCEDataEngine::POVDrawPeaks(CArchive* ar)
{
	int i;
	char szBuffer[400];

	for(i=0;i<PeakList.Natoms;i++)
	{

		sprintf(szBuffer,"sphere {< %f , %f , %f > , ",
				PeakList.a[i].cc.x,
				PeakList.a[i].cc.y,
				PeakList.a[i].cc.z);
		ar->WriteString(szBuffer);

		ar->WriteString("AtomRadiusForQ  pigment {color ColorForQ}}\n");
	}
}
