// MCEDataEngine.h: interface for the CMCEDataEngine class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MCEDATAENGINE_H__EA5AEAFB_40AE_4971_904F_A949F0ECD64D__INCLUDED_)
#define AFX_MCEDATAENGINE_H__EA5AEAFB_40AE_4971_904F_A949F0ECD64D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// coresponding to 20 Mb of memory in float 3D field
#define MAX_VOXEL_DIEMENSION  150

// nuber of maps witch could be loaded (now 2 Primary and secondary)
#define NMAPS 2 


//******************************************************************/
// Data structures for input data storage 

typedef struct{
float Start;
float Step;
float End;
float Division;
} AxisDescription;

typedef struct {
CString strMapFileName;
float PlaneMatrix[3][3];
float PlaneTranslation[3];
float Cell[6];
AxisDescription AxisParam[3];
int Lines;
int Columns;
int Sections;
// very non elegant non dynamic definition, but fast ...
float Voxel[MAX_VOXEL_DIEMENSION][MAX_VOXEL_DIEMENSION][MAX_VOXEL_DIEMENSION];
// we will determine following:
float maximum;
float minimum;
// ELD map visibility parameters ...
float layer_limits[3];
BOOL  layer_visibility[3];
// 2D ELD map visibility parameters ...
int map_2D_level;
BOOL  map_2D_visibility;
 // atom field could be aded later ...
} ELD_MapData;

/******************************************************************/
// Data structures related to the graphic generation

// displaylists symbolic names
// RESERVED = 0 , non correct name !!!

typedef enum {X,Y,Z} contour_direction;

typedef enum {RESERVED,MCE_TRIANGLES,MCE_CONTOURS,MCE_CONTOURS2D,MCE_MOLECULE} displayLists;

// GEMOETRICAL STRUCTURES
	struct XYZ
	{
	   float x;
	   float y;
       float z;     
	};

	typedef struct
	{
		XYZ p[2];
	} LINE;

	struct XY
	{
        float x;
        float y;    
	};

	typedef struct {
		XY p[2];
	} LINE2D;


	typedef struct {
		XY p[4];
		double val[4];
	} GRIDCELL2D;

	// my OpenGL color definition
	typedef struct
	{
		GLfloat R;
		GLfloat G;
		GLfloat B;
	} OPENGL_COLOR;

#define MAXCONTOURLINES  200000
//******************************************************************/
// Data structures for the molecule description
	#define MAXATOMS 5000
	#define MAXBONDS 6000

	typedef struct{
	char name[4]; // atom name
	XYZ fc; // fractional coordinates
	XYZ	cc;	// cartesian coordinates in given LS plane
	float u[6]; // thermal parameters
	float uiso; // isotropci thermal parameter
	float occ; // ocupancy
	int AtomNumber; // atom number in preiodic table
	int atm_label_number;  // atom number in the structure
	} AtomDescription;

	typedef struct{
	long nBond;
	int nIndex[MAXBONDS][2];
	} ConectivityMatrix;
	
	typedef struct{
	int Natoms;
	AtomDescription a[MAXATOMS];
	ConectivityMatrix bond;
	} MoleculeData;

	typedef struct{
	BOOL bInitialized;
	MoleculeData	Molecule;
	ELD_MapData MapData;
	// lattice parameters, symetry e.t.c could be added later
	} StructureDescription;

	
	typedef struct{
	// line controur field
	unsigned int iNcontourlines[3];
	LINE	contourlines[3][MAXCONTOURLINES];
	OPENGL_COLOR layer_color[3];
	} LineMesh;

//******************************************************************
//characteristic for individual elements

	typedef struct{
	int AtomNumber;
	float COVR;
	float GRAR;
	float VDWR;
	float Tolerance;
	char name[4]; // atom name
	float AtomWeight;
	} ElementDescription;

//******************************************************************
// general setup structure
typedef enum {Picking3D,Picking2D} pickingMode;
typedef enum {Perspective,Orthographic} perspectiveMode;

	typedef struct{

	BOOL bUnitCellVisible;
	BOOL bMoleculeVisible;
	pickingMode PickMode;
	perspectiveMode PerspMode;

	} GeneralSetup;

/******************************************************************/
// Class declaration ...
class CMCEDataEngine  
{
public:
	void SavePovRayInput();
	void InvertMatrix(float m[3][3],float mi[3][3]);
	void SavePeaks();
	void Addnewpeak();
	float Intensity(float x,float y,float z);
	XYZ MarkPosition;
	XYZ IndToCoord3Df(float x,float y,float z,ELD_MapData *map);
	BOOL bMarkVisible;
	BOOL m_bNeedRecalculateMap;
	BOOL m_bDataExist;
	void Scene();
	BOOL ReadFouFile(CString filename,int map_number);
	CMCEDataEngine();
	virtual ~CMCEDataEngine();
	
	// molecule parameters
	StructureDescription MolData[NMAPS]; // we can handle 2 modles simultanously
	MoleculeData PeakList;

	// Color parameters (should be extended later to atom,bonds e.t.c. ...)
	OPENGL_COLOR BackgrounColor;

	// Works setup parameters
	GeneralSetup Setup;

private:
	void POVDrawPeaks(CArchive* ar);
	void POVDrawUnitCell(CArchive* ar);
	void POVMceContours2D(CArchive *ar);
	void POVMceContours(CArchive* ar);
	void POVDrawAtoms(CArchive *ar);
	void POVcylinder(XYZ p1, XYZ p2, CString radius,CArchive *ar);
	void POVDrawBonds(CArchive* ar);
	void POVSaveMatrix(CArchive* ar);
	void GenerateLatticePoint(ELD_MapData *pMapData);
	void DrawUnitCell();
	XYZ CalcCartToFract(XYZ cc,float translation[],float matrix[][3]);
	void DrawPeaks();
	XYZ Coord3DfToInd(float x,float y,float z,ELD_MapData *MapData);
	void DrawMark();
	void DecodeAtomNames(MoleculeData *atoms);
	BOOL LoadInitialValues();
	void DrawBonds();
	void SetAtomColor(char name[2]);
	void DrawAtoms();
	float distance (XYZ p1, XYZ p2);
	void SetupConectivity(MoleculeData *atoms);
	void MultBy33Matrix(float v1[3],float v2[3],float matrix[3][3]);
	XYZ CalcFractToCart(XYZ fc,float translation[3],float matrix[3][3]);
	void StuctureToCart(MoleculeData *atoms, ELD_MapData *map);
	void GenerateDisplayLists();
	XY VertexInterp2D(double isolevel,XY p1,XY p2,double valp1,double valp2);
	int Polygonise2D(GRIDCELL2D grid,double isolevel,LINE2D *linefield);
	void VoxelDirectionalContour(ELD_MapData *map,contour_direction direction,LineMesh *pM);
	void VoxelDirectionalContour2D(ELD_MapData *map,contour_direction direction,LineMesh *pM);
	void GenerateContoursFromVoxel();
	void SetLayerLimitsFromVoxel(ELD_MapData *map);
	void OnRecalulateMap();
	void RenderScene();
	int m_nElementsNumber;
	ElementDescription m_ElementsProperties[200];
	LineMesh M[NMAPS]; // filed of the laer meshes ...
	LineMesh M2D[NMAPS]; // filed of the 2D layer meshes ...
	XYZ UnitCellEndPoints[8]; // end points of the model 0 unit cell ...
};

#endif // !defined(AFX_MCEDATAENGINE_H__EA5AEAFB_40AE_4971_904F_A949F0ECD64D__INCLUDED_)
