// MCEView.cpp : implementation of the CMCEView class
//

#include "stdafx.h"
#include "MCE.h"

#include "MCEDoc.h"
#include "MCEView.h"

// support loging function
#include "log.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const char* const CMCEView::_ErrorStrings[] = {
	{"No Error"}, // 0
	{"Unable to get a DC"}, // 1
	{"ChoosePixelFormat failed"}, // 2
	{"SelectPixelFormatFailed"}, // 3
	{"wglCreateContext failed"}, // 4
	{"wglMakeCurrent failed"}, // 5
	{"wglDelateContext failed"}, // 6
	{"SwapBuffers failed"}, // 7
};

/////////////////////////////////////////////////////////////////////////////
// CMCEView

IMPLEMENT_DYNCREATE(CMCEView, CView)

BEGIN_MESSAGE_MAP(CMCEView, CView)
	// custom modeless dialog message procesing
	ON_MESSAGE(WM_EXIT_LC, OnLevelControlExit)
	ON_MESSAGE(WM_APPLY_LC_CHANGES, OnApplyLevelControlChanges)
	ON_MESSAGE(WM_EXIT_LC_SEC, OnLevelControlExitSec)
	ON_MESSAGE(WM_APPLY_LC_CHANGES_SEC, OnApplyLevelControlChangesSec)
	//{{AFX_MSG_MAP(CMCEView)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_MOUSEMOVE()
	ON_WM_CANCELMODE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEWHEEL()
	ON_WM_RBUTTONDOWN()
	ON_WM_TIMER()
	ON_COMMAND(ID_VIEW_LEVELCONTROL, OnViewLevelControl)
	ON_COMMAND(ID_FILE_SAVE_BMP, OnFileSaveBmp)
	ON_COMMAND(ID_FILE_OPEN2, OnFileOpen2)
	ON_COMMAND(ID_VIEW_LEVELCONTROL_SEC, OnViewLevelControlSec)
	ON_COMMAND(ID_EDIT_ADDNEWPEAK, OnEditAddnewpeak)
	ON_COMMAND(ID_EDIT_CLEARPEAKLIST, OnEditClearpeaklist)
	ON_COMMAND(ID_PREFERENCES_ANIMATION, OnPreferencesAnimation)
	ON_UPDATE_COMMAND_UI(ID_PREFERENCES_ANIMATION, OnUpdatePreferencesAnimation)
	ON_UPDATE_COMMAND_UI(ID_EDIT_ADDNEWPEAK, OnUpdateEditAddnewpeak)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CLEARPEAKLIST, OnUpdateEditClearpeaklist)
	ON_UPDATE_COMMAND_UI(ID_VIEW_LEVELCONTROL, OnUpdateViewLevelcontrol)
	ON_UPDATE_COMMAND_UI(ID_VIEW_LEVELCONTROL_SEC, OnUpdateViewLevelcontrolSec)
	ON_COMMAND(ID_FILE_SAVEPEAKLIST, OnFileSavepeaklist)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVEPEAKLIST, OnUpdateFileSavepeaklist)
	ON_COMMAND(ID_PREFERENCES_BACKGROUNDCOLOR, OnPreferencesBackgroundcolor)
	ON_WM_KEYDOWN()
	ON_COMMAND(ID_PREFERENCES_GENERALSETUP, OnPreferencesGeneralSetup)
	ON_COMMAND(ID_FILE_SAVEPOVRAYINPUT, OnFileSavepovrayinput)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMCEView construction/destruction

CMCEView::CMCEView():
m_hRC(0), m_pDC(0), m_ErrorString(_ErrorStrings[0]),
ActualDisplayMode(NoStereo)
{
	// TODO: add construction code here

	// dialog classes inicialization
	// for non modal dialogs !!
	m_pDlgLevelControlPrim= new CMCELevelControlPrimary(this);
	m_pDlgLevelControlSec= new CMCELevelControlSecondary(this);

	// command line data processing
	CString str = AfxGetApp()->m_lpCmdLine;
	ProcessCommandLine(&str);
	
	// geometrical logic variables
	m_spinning = false;
	m_moving = false;
	m_scaling =false;
	m_animate=false;

	// trackball parameters
	W = 300;
	H = 300;

	scalefactor = 1.0;

	stretchX=1;
	stretchY=1;

	m_fshiftX=0;
	m_fshiftY=0;
	m_fshiftZ=0;

	// bakground color initialization has priority in this place
	
	CMCEDataEngine::BackgrounColor.R=0;
	CMCEDataEngine::BackgrounColor.G=0;
	CMCEDataEngine::BackgrounColor.B=0;
}

CMCEView::~CMCEView()
{
}

BOOL CMCEView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	
	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMCEView drawing

void CMCEView::OnDraw(CDC* pDC)
{
	CMCEDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here

	TRACE("CMCEView::OnDraw(CDC* pDC)\n");

	TestOpenGlError("OnDraw - begining");

	// reactivate the depth test
	glEnable (GL_DEPTH_TEST);

	switch (ActualDisplayMode)
	{
		case NoStereo:
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			Camera(CentralEye);
			RecalcModelView();
			RenderScene();
			glFinish();
			TestOpenGlError("OnDraw - NoStero");
			if ( FALSE == ::SwapBuffers (m_pDC -> GetSafeHdc() ) )
			{
				SetError(7);
			}
		break;

		case StereoHW:
			//Left Image
			glDrawBuffer (GL_BACK_LEFT);
			glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			Camera(LeftEye);
	        RecalcModelView();
			RenderScene();
			// Right Image
			glDrawBuffer (GL_BACK_RIGHT);
			glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			Camera (RightEye);
			RecalcModelView();
			RenderScene();
			glFinish();
			TestOpenGlError("OnDraw - SteroHW");
			if ( FALSE == ::SwapBuffers (m_pDC -> GetSafeHdc() ) )
			{
				SetError(7);
			}
		break;

		case StereoInterlaced:
		case StereoVertical:
    
			glEnable (GL_STENCIL_TEST);
			glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
              
			//Left Image
			glStencilFunc (GL_NOTEQUAL, 1, 1); // draws if stencil <> 1
			Camera (LeftEye);
			RecalcModelView();
			RenderScene();
			//Right Image
			glStencilFunc (GL_EQUAL, 1, 1); // draws if stencil <> 0
			Camera (RightEye);
			RecalcModelView();
			RenderScene();
			glFinish();
			TestOpenGlError("OnDraw - SteroInterlaced");
			if ( FALSE == ::SwapBuffers (m_pDC -> GetSafeHdc() ) )
			{
				SetError(7);
			}
        
		break;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CMCEView diagnostics

#ifdef _DEBUG
void CMCEView::AssertValid() const
{
	CView::AssertValid();
}

void CMCEView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CMCEDoc* CMCEView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMCEDoc)));
	return (CMCEDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMCEView message handlers

void CMCEView::OnFileOpen() 
{
	CFileDialog dlg(TRUE, NULL, "Untitled", OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "Asci FOU Files (*.fou)|*.fou|All Files (*.*)|*.*||");
 
	if(dlg.DoModal() == IDOK)
	{
		// TODO: Add new data handling		
		TRACE("File name (primary) : %s\n",dlg.GetPathName());
		CMCEDataEngine::ReadFouFile(dlg.GetPathName(),0);
	}
	
  // we need inmediate redraw for new structure ...

  Invalidate(TRUE);
}

void CMCEView::OnFileOpen2() 
{
	CFileDialog dlg(TRUE, NULL, "Untitled", OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "Asci FOU Files (*.fou)|*.fou|All Files (*.*)|*.*||");
 
	if(dlg.DoModal() == IDOK)
	{
		// TODO: Add new data handling		
		TRACE("File name (secondary): %s\n",dlg.GetPathName());
		CMCEDataEngine::ReadFouFile(dlg.GetPathName(),1);
	}
	
  // we need inmediate redraw for new structure ...

  Invalidate(TRUE);
}

int CMCEView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here

	InitializeOpenGL();

	// timer setup for animation
	SetTimer(1,33,NULL); // max 30 fps
	
	return 0;
}

void CMCEView::OnDestroy() 
{
	CView::OnDestroy();
	
	// TODO: Add your message handler code here

	if ( FALSE == ::wglMakeCurrent(0,0))
	{
		SetError(2);
	}

	if ( FALSE == ::wglDeleteContext( m_hRC) )
	{
		SetError(6);
	}
	
	if ( m_pDC )
	{
		delete m_pDC;
	}
}

BOOL CMCEView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default

	// comment out original call
	// return CView::OnEraseBkgnd(pDC);
	return TRUE; // tell Windows not to erase the background
}

void CMCEView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here	

	if( 0>= cx || 0 >= cy)
	{
		return; // to avoid nonsence calls
	}

	SetupViewport (cx,cy);

	stretchX = 1;
    stretchY = 1;
     
    if (((float)cy / (float)cx) > 1)
	{
		stretchY = (float)cy /(float)cx;
	}

    if (((float)cy /(float) cx) < 1) 
	{
		stretchX = (float)cx /(float)cy;
	}

	// TODO add the rest of the geometry operation in this place
}

/////////////////////////////////////////////////////////////////////////////
// OpenGL initialization functions

void CMCEView::SetError( int e )
{
	// if there was no previous error,
	// then save this one
	if ( _ErrorStrings[0] == m_ErrorString )
	{
		m_ErrorString = _ErrorStrings[e];
	}

	// we will always write such critical error
	AfxMessageBox(_ErrorStrings[e]);
}


BOOL CMCEView::InitializeOpenGL()
{
	m_pDC = new CClientDC(this);

	if ( NULL == m_pDC ) // failure to get DC
	{
		SetError(1);
		return FALSE;
	}

	if ( !SetupPixelFormat() )
	{
		return FALSE;
	}

	if ( 0 == (m_hRC =
		::wglCreateContext ( m_pDC->GetSafeHdc() ) ) )
	{
		SetError(4);
		return FALSE;
	}

	if ( FALSE ==
		::wglMakeCurrent( m_pDC->GetSafeHdc() , m_hRC) )
	{
		SetError(5);
		return FALSE;
	}

	// Other OS non-specific nessesry initializations
	CMCEView::SupportInitializations();

	return TRUE;
}

BOOL CMCEView::SetupPixelFormat()
{

	DWORD dwFlags;
	int iStencil;
	
	switch (ActualDisplayMode)
	{
		case NoStereo:
			dwFlags =
			PFD_DRAW_TO_WINDOW |	// support window
			PFD_SUPPORT_OPENGL |	// support OpenGL
			PFD_DOUBLEBUFFER ;		// double buffered
			iStencil =0;			// stencil buffer not required	
		break;

		case StereoHW:
			dwFlags =
			PFD_DRAW_TO_WINDOW |	// support window
			PFD_SUPPORT_OPENGL |	// support OpenGL
			PFD_STEREO |			// activate stereoscopic mode
			PFD_DOUBLEBUFFER ;		// double buffered
			iStencil =0;			// stencil buffer not required	
		break;

		case StereoInterlaced:
		case StereoVertical:
			dwFlags =
			PFD_DRAW_TO_WINDOW |	// support window
			PFD_SUPPORT_OPENGL |	// support OpenGL
			PFD_DOUBLEBUFFER ;		// double buffered        
			iStencil =1;			// necessary	
		break;
	}

	PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),// size of this pfd
		1,						// version number
		dwFlags ,				// device setup
		PFD_TYPE_RGBA,			// RGBA type
		24,						// 24-bit color depth
		0,0,0,0,0,0,			// color bits ignored
		0,						// no alpha buffer
		0,						// shift bit ignored
		0,						// no accumulation buffer
		0,0,0,0,				// accum bits ignored
		16,						// 16 bit z-buffer
		iStencil,				// no stencil buffer
		0,						// no auxiliary buffer
		PFD_MAIN_PLANE,			// main layer
		0,						// reserved
		0,0,0					// layer mask ignored
	};

	int pixelformat;

	if ( 0 == (pixelformat = 
	::ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)))
	{
		SetError(2);
		return FALSE;
	}
	
	if ( FALSE == ::SetPixelFormat(m_pDC->GetSafeHdc(),
		pixelformat,&pfd) )
	{
		SetError(3);
		return FALSE;
	}

	return TRUE;
}

BOOL CMCEView::RenderScene()
{
	glPushMatrix();	
		CMCEDataEngine::Scene();
	glPopMatrix();

	return TRUE ;
}


/////////////////////////////////////////////////////////////////////
// COpenGLView helper functions	

BOOL CMCEView::SetupViewport(int cx, int cy) // GLint declaration shud be better
{
	// select the full client area
	::glViewport(0,0, cx, cy);

	if(ActualDisplayMode==StereoInterlaced)
	{
		interlace_stencil_horizontal(cx,cy);
	}

	if(ActualDisplayMode==StereoVertical)
	{
		interlace_stencil_vertical(cx,cy);
	}

	// code from dinospin
	W = cx;
	H = cy;
	return TRUE;
}

bool CMCEView::TestOpenGlError(const char * szMessage)
{
	GLenum errCode;
	const GLubyte *errString;
	char szBuffer[256];

		// OpenGl error examination
	if((errCode= glGetError())!=GL_NO_ERROR)
	{
		errString=gluErrorString(errCode);
		sprintf(szBuffer,"OpenGl Error: %s\n Error location: %s",errString,szMessage);
		TRACE(szBuffer);
		AfxMessageBox(szBuffer);
		return false;
	}
	else
	{
		return true;
	}
}

void CMCEView::Camera(Eye mode)
{
	float a;
	float z;
	float dist;
	
	a = 6; // visible part of the space
	z = 15; // Z depth of the space
	dist = z * (float)1.5;// distance from center of projection to zeroplane
	switch(Setup.PerspMode)
	{
		case Perspective:
			switch (mode)
			{
				case CentralEyePicking:
					CMCESupportRoutines::stereo_projection_picking(-a * stretchX, a * stretchX, -a * stretchY, a * stretchY, z, -z, 0, dist, 0); 
				break;

				case CentralEye:
					CMCESupportRoutines::stereo_projection(-a * stretchX, a * stretchX, -a * stretchY, a * stretchY, z, -z, 0, dist, 0); 
				break;

				case LeftEye:
					CMCESupportRoutines::stereo_projection(-a * stretchX, a * stretchX, -a * stretchY, a * stretchY, z, -z, 0, dist, (float)-0.31); //lef eye
				break;

				case RightEye:
					CMCESupportRoutines::stereo_projection(-a * stretchX, a * stretchX, -a * stretchY, a * stretchY, z, -z, 0, dist, (float)0.31); //same as lef eye positive
				break;
			}   
		break;

		case Orthographic:
			switch (mode)
			{
				case CentralEyePicking:
					CMCESupportRoutines::ortho_projection_picking(-a * stretchX, a * stretchX, -a * stretchY, a * stretchY, z, -z, 0, dist, 0); 
				break;

				default:
					CMCESupportRoutines::ortho_projection(-a * stretchX, a * stretchX, -a * stretchY, a * stretchY, z, -z, 0, dist, 0); 
				break;
			}   
		break;
	}
}

void CMCEView::RecalcModelView()
{
  GLfloat m[4][4];

  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity();

  CTrackball::build_rotmatrix(m, curquat);
  glMultMatrixf(&m[0][0]);

  if (scalefactor == 1.0) {
    glDisable(GL_NORMALIZE);
  } else {
    glEnable(GL_NORMALIZE);
  }

  glTranslatef(m_fshiftX,m_fshiftY,m_fshiftZ);

  glScalef(scalefactor, scalefactor, scalefactor);

}

void CMCEView::SupportInitializations()
{
	// specify black as clear color
	glClearColor( BackgrounColor.R,BackgrounColor.G,BackgrounColor.B,0 );
	// specify the back of the buffer as clera depth
	glClearDepth( 1.0f);
	// enable depth testing
	glEnable( GL_DEPTH_TEST);

	// setup lighting
	GLfloat light_position[]={1.0,1.0,1.0,0.0};
	glLightfv(GL_LIGHT0,GL_POSITION,light_position);
	glEnable(GL_LIGHT0);

	// makes colored atoms possible ...
	glEnable (GL_COLOR_MATERIAL);	// color surfaces of materials
	glShadeModel(GL_SMOOTH);		// Enables Smooth Color Shading

	// initialize vieweing matrix ...
	CTrackball::trackball(curquat, 0.0, 0.0, 0.0, 0.0);
}

void CMCEView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnMouseMove(nFlags, point);

	if (m_scaling) {
    scalefactor =(float)( scalefactor * (1.0 + (((float) ((float)beginy - (float)point.y)) / H)));
    beginx = point.x;
    beginy = point.y;
	CMCEView::Animate();
    return;
	}

  if (m_moving)
  {
	  CTrackball::trackball(lastquat,
      (float)(2.0 * beginx - W) / W,
      (float)(H - 2.0 * beginy) / H,
      (float)(2.0 * (float)point.x - W) / W,
      (float)(H - 2.0 * (float)point.y) / H
      );
    beginx = point.x;
    beginy = point.y;
    m_spinning = true;
    //glutIdleFunc(animate);
	CMCEView::Animate();
  }
}

void CMCEView::OnCancelMode() 
{
	CView::OnCancelMode();
	
	// TODO: Add your message handler code here
	
}

void CMCEView::Animate()
{
	TRACE("Animate()\n");

if ((! m_scaling) && m_spinning) // rotation ony for non-scaling mode
{
  add_quats(lastquat, curquat, curquat);
}
 
  // elegant method of redrawing ....
  Invalidate(TRUE);

}

void CMCEView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnLButtonDown(nFlags, point);

		// TODO: Add your message handler code here and/or call default
	// TODO: Add your message handler code here and/or call default
	
	CView::OnLButtonDown(nFlags, point);

	// redirect mouse messages to this pint from outside window
	SetCapture();

    m_spinning = false;
    m_moving = true;
    beginx = point.x;
    beginy = point.y;

	if ((nFlags & MK_SHIFT)!=0) {
      m_scaling = true;
    } else {
      m_scaling = false;
    }

}

void CMCEView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnLButtonUp(nFlags, point);
	
	// stop the outside window mouse tracing
	ReleaseCapture();

    m_moving = false;
	m_scaling= false;

}

BOOL CMCEView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	// TODO: Add your message handler code here and/or call default
	
	bool backup_moving;
	bool backup_scaling;

	backup_moving=m_moving;
	backup_scaling=m_scaling;

	m_moving=false;
	m_scaling=true;
	
	if (zDelta > 0)
	{
		scalefactor =(float)( scalefactor * 1.1);
	}
	else
	{
		scalefactor =(float)( scalefactor * 0.9);
	}
 
	if (m_animate==false) CMCEView::Animate();

	m_moving=backup_moving;
	m_scaling=backup_scaling;
	
	return CView::OnMouseWheel(nFlags, zDelta, pt);
}

void CMCEView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnRButtonDown(nFlags, point);

// click projection into the map

GLint viewport[4];
GLdouble mvmatrix[16], projmatrix [16];
GLint realy; // OpenGl y coordinate
int x,y; // click position
GLdouble wx,wy,wz; // returned model coordinates
char szBuffer[250];
float z; // cycle parameter ...

	x=point.x;
	y=point.y;

	glGetIntegerv(GL_VIEWPORT, viewport);
	glGetDoublev(GL_MODELVIEW_MATRIX, mvmatrix);
	glGetDoublev(GL_PROJECTION_MATRIX, projmatrix);

	realy=viewport[3]-(GLint)y-1;

	sprintf(szBuffer,"Click postion x %d y %d\n",x,realy);
	LogMsg(szBuffer);

	gluUnProject((GLdouble)x,(GLdouble) realy, 0.5,
	mvmatrix,projmatrix,viewport,&wx,&wy,&wz);

	sprintf(szBuffer,"World coords at z=0.5 x %f y %f z %f\n",
			wx,wy,wz);
	LogMsg(szBuffer);

	float IntMax=-1000;
	float Intens;
	XYZ MaxPosition;

	switch(Setup.PickMode)
	{
		case Picking3D:
		for(z=0; z<=1.0; z=z+0.001)
		{
			gluUnProject((GLdouble)x,(GLdouble) realy, z,
			mvmatrix,projmatrix,viewport,&wx,&wy,&wz);
			Intens=Intensity(wx,wy,wz);
			if(Intens> IntMax)
			{
				IntMax=Intens;
				MaxPosition.x=wx;
				MaxPosition.y=wy;
				MaxPosition.z=wz;
			}
		}
		break;

		case Picking2D:
			// we will find the plane z

			XYZ p0=CMCEDataEngine::IndToCoord3Df(0,0,MolData[0].MapData.map_2D_level, &MolData[0].MapData);
			GLdouble z3=p0.z;
			GLdouble x1,y1,z1,x2,y2,z2;

			// we will find eqation of a line in space from 2 points ...

			gluUnProject((GLdouble)x,(GLdouble) realy, 0.0,
				mvmatrix,projmatrix,viewport,&x1,&y1,&z1);

			gluUnProject((GLdouble)x,(GLdouble) realy, 1.0,
				mvmatrix,projmatrix,viewport,&x2,&y2,&z2);

			// coeficient for the line equation
			float lambda=(z3-z1)/(z3-z2);

			MaxPosition.x=(x1-lambda*x2)/(1-lambda);
			MaxPosition.y=(y1-lambda*y2)/(1-lambda);
			MaxPosition.z=z3;

		break;
	}


	MarkPosition=MaxPosition;

	bMarkVisible=TRUE;
	 
	// elegant method of redrawing ....
	Invalidate(TRUE);

}

void CMCEView::OnTimer(UINT nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
	
	if (m_animate)
	{
		CMCEView::Animate();
	}
	
	CView::OnTimer(nIDEvent);
}

void CMCEView::OnViewLevelControl() 
{
	// TODO: Add your command handler code here
	// creates the dialog if not created already

	m_pDlgLevelControlPrim->m_fMapMinimum=MolData[0].MapData.minimum;
	m_pDlgLevelControlPrim->m_fMapMaximum=MolData[0].MapData.maximum;
	// limits
	m_pDlgLevelControlPrim->m_fLayer1Value=MolData[0].MapData.layer_limits[0];
	m_pDlgLevelControlPrim->m_fLayer2Value=MolData[0].MapData.layer_limits[1];
	m_pDlgLevelControlPrim->m_fLayer3Value=MolData[0].MapData.layer_limits[2];
	// visibility
	m_pDlgLevelControlPrim->m_bLayer1Visible=MolData[0].MapData.layer_visibility[0];
	m_pDlgLevelControlPrim->m_bLayer2Visible=MolData[0].MapData.layer_visibility[1];
	m_pDlgLevelControlPrim->m_bLayer3Visible=MolData[0].MapData.layer_visibility[2];

	// 2D map parameters
	m_pDlgLevelControlPrim->m_n2D_map_level=MolData[0].MapData.map_2D_level;
	m_pDlgLevelControlPrim->m_b2D_map_Visible=MolData[0].MapData.map_2D_visibility;


	if (m_pDlgLevelControlPrim->GetSafeHwnd() == 0) {
		m_pDlgLevelControlPrim->Create(); // displays the dialog window
	}

	m_pDlgLevelControlPrim->m_wndSpin_2D_Level.SetRange(0,MolData[0].MapData.Sections-1);
	
}

void CMCEView::OnViewLevelControlSec() 
{
	// TODO: Add your command handler code here
	// creates the dialog if not created already

	m_pDlgLevelControlSec->m_fMapMinimum=MolData[1].MapData.minimum;
	m_pDlgLevelControlSec->m_fMapMaximum=MolData[1].MapData.maximum;
	// limits
	m_pDlgLevelControlSec->m_fLayer1Value=MolData[1].MapData.layer_limits[0];
	m_pDlgLevelControlSec->m_fLayer2Value=MolData[1].MapData.layer_limits[1];
	m_pDlgLevelControlSec->m_fLayer3Value=MolData[1].MapData.layer_limits[2];
	// visibility
	m_pDlgLevelControlSec->m_bLayer1Visible=MolData[1].MapData.layer_visibility[0];
	m_pDlgLevelControlSec->m_bLayer2Visible=MolData[1].MapData.layer_visibility[1];
	m_pDlgLevelControlSec->m_bLayer3Visible=MolData[1].MapData.layer_visibility[2];

		// 2D map parameters
	m_pDlgLevelControlSec->m_n2D_map_level=MolData[1].MapData.map_2D_level;
	m_pDlgLevelControlSec->m_b2D_map_Visible=MolData[1].MapData.map_2D_visibility;


	if (m_pDlgLevelControlSec->GetSafeHwnd() == 0) {
		m_pDlgLevelControlSec->Create(); // displays the dialog window
	}

	m_pDlgLevelControlSec->m_wndSpin_2D_Level.SetRange(0,MolData[1].MapData.Sections-1);

}


LRESULT CMCEView::OnLevelControlExit(WPARAM wParam, LPARAM lParam)
{
	// message received in response to modeless dialog OK
	//  and Cancel buttons
	TRACE("CMCEView::OnLevelControlExit %x, %lx\n", wParam, lParam);
	TRACE("L1 %d L2 %d L3 %d\n",
	m_pDlgLevelControlPrim->m_nLayer1Scroll,
	m_pDlgLevelControlPrim->m_nLayer2Scroll,
	m_pDlgLevelControlPrim->m_nLayer3Scroll);

	m_pDlgLevelControlPrim->DestroyWindow();
	return 0L;
}

LRESULT CMCEView::OnLevelControlExitSec(WPARAM wParam, LPARAM lParam)
{
	// message received in response to modeless dialog OK
	//  and Cancel buttons
	TRACE("CMCEView::OnLevelControlExitSec %x, %lx\n", wParam, lParam);
	TRACE("L1 %d L2 %d L3 %d\n",
	m_pDlgLevelControlSec->m_nLayer1Scroll,
	m_pDlgLevelControlSec->m_nLayer2Scroll,
	m_pDlgLevelControlSec->m_nLayer3Scroll);

	m_pDlgLevelControlSec->DestroyWindow();
	return 0L;
}


LRESULT CMCEView::OnApplyLevelControlChanges(WPARAM wParam, LPARAM lParam)
{
	// graphic refresh request from contol dialog

	// limits
	MolData[0].MapData.layer_limits[0]=m_pDlgLevelControlPrim->m_fLayer1Value;
	MolData[0].MapData.layer_limits[1]=m_pDlgLevelControlPrim->m_fLayer2Value;
	MolData[0].MapData.layer_limits[2]=m_pDlgLevelControlPrim->m_fLayer3Value;
	// visibility
	MolData[0].MapData.layer_visibility[0]=m_pDlgLevelControlPrim->m_bLayer1Visible;
	MolData[0].MapData.layer_visibility[1]=m_pDlgLevelControlPrim->m_bLayer2Visible;
	MolData[0].MapData.layer_visibility[2]=m_pDlgLevelControlPrim->m_bLayer3Visible;

	// 2D map parameters
	MolData[0].MapData.map_2D_visibility=m_pDlgLevelControlPrim->m_b2D_map_Visible;
	MolData[0].MapData.map_2D_level=m_pDlgLevelControlPrim->m_n2D_map_level;

	CMCEDataEngine::m_bNeedRecalculateMap=TRUE;

	Invalidate(TRUE);

	return 0L;
}

LRESULT CMCEView::OnApplyLevelControlChangesSec(WPARAM wParam, LPARAM lParam)
{
	// graphic refresh request from contol dialog

	// limits
	MolData[1].MapData.layer_limits[0]=m_pDlgLevelControlSec->m_fLayer1Value;
	MolData[1].MapData.layer_limits[1]=m_pDlgLevelControlSec->m_fLayer2Value;
	MolData[1].MapData.layer_limits[2]=m_pDlgLevelControlSec->m_fLayer3Value;
	// visibility
	MolData[1].MapData.layer_visibility[0]=m_pDlgLevelControlSec->m_bLayer1Visible;
	MolData[1].MapData.layer_visibility[1]=m_pDlgLevelControlSec->m_bLayer2Visible;
	MolData[1].MapData.layer_visibility[2]=m_pDlgLevelControlSec->m_bLayer3Visible;

	// 2D map parameters
	MolData[1].MapData.map_2D_visibility=m_pDlgLevelControlSec->m_b2D_map_Visible;
	MolData[1].MapData.map_2D_level=m_pDlgLevelControlSec->m_n2D_map_level;

	CMCEDataEngine::m_bNeedRecalculateMap=TRUE;

	Invalidate(TRUE);

	return 0L;
}

void CMCEView::OnFileSaveBmp() 
{
	// TODO: Add your command handler code here
	// TODO: Add your message handler code here and/or call default

	CMCEPictureSetup dlg;

	dlg.m_nPictureWidth=640;
	dlg.m_nPictureHeight=480;


	int ret =dlg.DoModal();

	TRACE("DoModal return =%d\n",ret);

	TRACE("Width: %d Height: %d\n",dlg.m_nPictureWidth,dlg.m_nPictureHeight);

	CMCEView::SaveBMP(dlg.m_nPictureWidth, dlg.m_nPictureHeight);	
}

void CMCEView::SaveBMP(int Width, int Height)
{
	CFileDialog dlg(FALSE, "bmp", "MCEimage.bmp", OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, "Bitmap Files (*.bmp)|*.bmp|All Files (*.*)|*.*| |");

	HDC dc;
	BITMAPINFO info;
	BITMAPFILEHEADER header;
	HBITMAP bitmap;
	GLubyte *bits;
	PIXELFORMATDESCRIPTOR pfd;
	HGLRC rc;
	CRect rect;
	CRect window_rect;
	CFile cfBitmap;
	CDC m_dc;

	int pf;

	long ImageAreaLenght;
	DisplayMode DisplayModeBackup;

	// backup of the eventualy non/stereoscopic mode output
	DisplayModeBackup = ActualDisplayMode;

	if(dlg.DoModal() == IDOK)
	{
		m_dc.CreateCompatibleDC(NULL);
		dc = m_dc.GetSafeHdc();

		memset(&info, 0, sizeof(info));
		info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
		info.bmiHeader.biWidth = Width;
		info.bmiHeader.biHeight = Height;
        info.bmiHeader.biPlanes = 1;
        info.bmiHeader.biBitCount = 24;
        info.bmiHeader.biCompression = BI_RGB;

        bitmap = CreateDIBSection(dc, &info, DIB_RGB_COLORS, (void**)&bits, NULL, 0);

        SelectObject(dc, bitmap);

        memset(&pfd, 0, sizeof(pfd));
        pfd.nSize = sizeof(pfd);
        pfd.nVersion = 1;
        pfd.dwFlags = PFD_DRAW_TO_BITMAP | PFD_SUPPORT_OPENGL ;
        pfd.iPixelType = PFD_TYPE_RGBA;
        pfd.cColorBits = 24;
        pfd.cRedBits = 8;
        pfd.cBlueBits = 8;
        pfd.cGreenBits = 8;
        pfd.cDepthBits = 16;

        pf = ChoosePixelFormat(dc, &pfd);
        SetPixelFormat(dc, pf, &pfd);

        rc = wglCreateContext(dc);
        wglMakeCurrent(dc, rc); 
		
		// We must reinitialize the OpenGL setup
		// We can not call SupportInitializations();
		// becouse we do not want to reset the rotations ...

	// specify black as clear color
	glClearColor( BackgrounColor.R,BackgrounColor.G,BackgrounColor.B,0 );
	// specify the back of the buffer as clera depth
	glClearDepth( 1.0f);
	// enable depth testing
	glEnable( GL_DEPTH_TEST);

	// setup lighting
	GLfloat light_position[]={1.0,1.0,1.0,0.0};
	glLightfv(GL_LIGHT0,GL_POSITION,light_position);
	glEnable(GL_LIGHT0);

	// makes colored atoms possible ...
	glEnable (GL_COLOR_MATERIAL);	// color surfaces of materials
	glShadeModel(GL_SMOOTH);		// Enables Smooth Color Shading
		
		// On Size with the output frame dimension
		CMCEView::OnSize(NULL, Width, Height); 
	
		// We must restore the display list after OpenGl reset !!
		// wery hard to find error :-( 

		CMCEDataEngine::m_bNeedRecalculateMap=TRUE;
		
		// my output code
		//COpenGLView::SetupViewingTransform();
		//COpenGLView::InitializeOpenGL_common();
		
		ActualDisplayMode = NoStereo;
		// OnDraw with the BMP m_dc
		CMCEView::OnDraw(&m_dc);		
		//CMCEView::RenderScene();		

        // Save here
        header.bfType = 'MB';
        header.bfSize = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + (info.bmiHeader.biWidth *
						info.bmiHeader.biBitCount + 7) / 8 * info.bmiHeader.biHeight;
        header.bfReserved1 = 0;
        header.bfReserved2 = 0;
        header.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);

		TRACE("cfBitmap.Open\n");
        cfBitmap.Open(dlg.GetPathName(), CFile::modeCreate | CFile::modeWrite | CFile::typeBinary);
		TRACE("cfBitmap.Write(&header, sizeof(BITMAPFILEHEADER))\n");
        cfBitmap.Write(&header, sizeof(BITMAPFILEHEADER));
		TRACE("cfBitmap.Write(&info, sizeof(BITMAPINFOHEADER))\n");     
        cfBitmap.Write(&info, sizeof(BITMAPINFOHEADER));
		
		//MH this command couse crash for 640x480 resolution
		TRACE("cfBitmap.Write(bits.....\n");
		
		// Mysaksoft Modification

		ImageAreaLenght=((info.bmiHeader.biWidth * info.bmiHeader.biBitCount + 7) / 8 * info.bmiHeader.biHeight);

		TRACE("ImageAreaLenght %d \n",ImageAreaLenght);

		TRACE("info.bmiHeader.biWidth %d info.bmiHeader.biBitCount %d info.bmiHeader.biHeight %d\n",info.bmiHeader.biWidth,info.bmiHeader.biBitCount,info.bmiHeader.biHeight);

		TRACE("sizeof(BITMAPFILEHEADER) %d sizeof(BITMAPINFOHEADER) %d)",sizeof(BITMAPFILEHEADER),sizeof(BITMAPINFOHEADER));


		// bug in original oced identified by MH
        //cfBitmap.Write(bits, sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + ImageAreaLenght);

		// correction
		cfBitmap.Write(bits,ImageAreaLenght);

        TRACE("cfBitmap.Close()...\n");  
		cfBitmap.Close();

        wglDeleteContext(rc);
        DeleteObject(bitmap);
        m_dc.DeleteDC();

		// reseting to screen output
        wglMakeCurrent(m_pDC->GetSafeHdc(), m_hRC);
		
		// screen redraw
        Invalidate(TRUE);

		// We must restore the H and W values
		// for the proper trackball work !!
		CMCEView::GetClientRect(window_rect);
		W=window_rect.right;
		H=window_rect.bottom;

		// we must restore the viewport in adition as well
		CMCEView::OnSize(NULL, W, H); 

		// restoring the original output setup as well 
		ActualDisplayMode = DisplayModeBackup;

		TRACE("End PrintBmp: W: %d H: %d \n",W,H);	
	}
}


void CMCEView::OnEditAddnewpeak() 
{
	// TODO: Add your command handler code here
	CMCEDataEngine::Addnewpeak();

	bMarkVisible=FALSE;

	Invalidate(TRUE);
	
}

void CMCEView::OnEditClearpeaklist() 
{
	// TODO: Add your command handler code here

	PeakList.Natoms=0;
	Invalidate(TRUE);
	
}

void CMCEView::OnPreferencesAnimation() 
{
	// TODO: Add your command handler code here

	CMenu* pMenu = GetMenu();
	
// animation switch option ....
	if(m_animate)
	{
		m_animate=false;
	}
	else
	{
		m_animate=true;
	}	
}



void CMCEView::OnUpdatePreferencesAnimation(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(m_animate);
}	

void CMCEView::OnUpdateEditAddnewpeak(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(bMarkVisible);
	
}

void CMCEView::OnUpdateEditClearpeaklist(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	pCmdUI->Enable(PeakList.Natoms>0);	
}

void CMCEView::OnUpdateViewLevelcontrol(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(MolData[0].bInitialized);
	
}

void CMCEView::OnUpdateViewLevelcontrolSec(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(MolData[1].bInitialized);	
}

void CMCEView::OnFileSavepeaklist() 
{
	// TODO: Add your command handler code here
	CMCEDataEngine::SavePeaks();	
}

void CMCEView::OnUpdateFileSavepeaklist(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here

	pCmdUI->Enable(PeakList.Natoms>0);
	
}

void CMCEView::OnPreferencesBackgroundcolor() 
{
	// TODO: Add your command handler code here
		// TODO: Add your command handler code here
	CColorDialog dlg;
	COLORREF color;
	bool backup_animate;

	backup_animate=m_animate;
	m_animate=false;

	if(dlg.DoModal() == IDOK)
	{
		color=dlg.GetColor();
		BackgrounColor.B=((float)(((color) >> 16) & 0xff))/255;
		BackgrounColor.G=((float)(((color) >> 8) & 0xff))/255;
		BackgrounColor.R=((float)((color) & 0xff))/255;
	
		glClearColor( BackgrounColor.R, BackgrounColor.G, BackgrounColor.B, 0.0f );
		
		Invalidate(TRUE);
	}

	TRACE("red %d green %d blue %d \n",BackgrounColor.R,BackgrounColor.G,BackgrounColor.B);

	m_animate=backup_animate;
	
}

void CMCEView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default

	//
	// Move  when the left, right, up and down (home) key is pressed.

	GLfloat m[4][4];
	GLfloat mred[3][3];
	GLfloat mi[3][3];
	const float shift_step=0.5;

	CTrackball::build_rotmatrix(m, curquat);

	for(int i=0;i<3;i++)
	{
		for(int j=0;j<3;j++)
		{
			mred[i][j]=m[i][j]; // reduction to 3x3 matrix
		}
	}

	InvertMatrix(mred, mi);

    switch (nChar) {

		// corrected J.C. equation for translation
		// reflection correct InvertMatrix calculation ...

		case VK_LEFT:
			m_fshiftX = m_fshiftX - mi[0][0] * shift_step;
			m_fshiftY = m_fshiftY - mi[0][1] * shift_step;
			m_fshiftZ = m_fshiftZ - mi[0][2] * shift_step;
			Invalidate(TRUE);
        break;

		case VK_RIGHT:
			m_fshiftX = m_fshiftX + mi[0][0] * shift_step;
			m_fshiftY = m_fshiftY + mi[0][1] * shift_step;
			m_fshiftZ = m_fshiftZ + mi[0][2] * shift_step;
			Invalidate(TRUE);
        break;

		case VK_UP:
			m_fshiftX = m_fshiftX + mi[1][0] * shift_step;
			m_fshiftY = m_fshiftY + mi[1][1] * shift_step;
			m_fshiftZ = m_fshiftZ + mi[1][2] * shift_step;
			Invalidate(TRUE);
        break;

	    case VK_DOWN:
			m_fshiftX = m_fshiftX - mi[1][0] * shift_step;
			m_fshiftY = m_fshiftY - mi[1][1] * shift_step;
			m_fshiftZ = m_fshiftZ - mi[1][2] * shift_step;
			Invalidate(TRUE);
        break;

		case 'A':
		case 'a':
			m_fshiftX = m_fshiftX + mi[2][0] * shift_step;
			m_fshiftY = m_fshiftY + mi[2][1] * shift_step;
			m_fshiftZ = m_fshiftZ + mi[2][2] * shift_step;
			Invalidate(TRUE);
        break;

		case 'Z':
		case 'z':
			m_fshiftX = m_fshiftX - mi[2][0] * shift_step;
			m_fshiftY = m_fshiftY - mi[2][1] * shift_step;
			m_fshiftZ = m_fshiftZ - mi[2][2] * shift_step;
			Invalidate(TRUE);
        break;

		case VK_HOME:
			m_fshiftX=0;
			m_fshiftY=0;
			m_fshiftZ=0;
			Invalidate(TRUE);
        break;
    }
	
//	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CMCEView::OnPreferencesGeneralSetup() 
{
	// TODO: Add your command handler code here

		// TODO: Add your command handler code here
		// TODO: Add your command handler code here
	CMCEGeneralSetup dlg;
	bool backup_animate;

	backup_animate=m_animate;
	m_animate=false;

	dlg.m_bMoleculeVisible=Setup.bMoleculeVisible;
	dlg.m_bUnitCellVisible=Setup.bUnitCellVisible;
	dlg.m_iPerspMode=Setup.PerspMode;
	dlg.m_iPickMode=Setup.PickMode;

	if(dlg.DoModal() == IDOK)
	{
		Setup.bMoleculeVisible=dlg.m_bMoleculeVisible;
		Setup.bUnitCellVisible=dlg.m_bUnitCellVisible;
		Setup.PerspMode=(perspectiveMode)dlg.m_iPerspMode;
		Setup.PickMode=(pickingMode)dlg.m_iPickMode;
		Invalidate(TRUE);
	}

	m_animate=backup_animate;	
}

void CMCEView::OnFileSavepovrayinput() 
{
	// TODO: Add your command handler code here

	CMCEDataEngine::SavePovRayInput();		

}

void CMCEView::ProcessCommandLine(CString *str)
{
	str->TrimLeft(); str->TrimRight();
	if(!str->IsEmpty()) 
	{
		// TODO proces command line and activate the data
		TRACE("Command line string: %s\n",str);

		if((str->Left(1)=='"')&&(str->Right(1)=='"'))
		{
			// W2k command line string with " on and and beggining
			str->TrimLeft('"');
			str->TrimRight('"');
		}
		
		// switch to HW page flipping stereo mode
		if(str->Left(2)=="-s")
		{
			ActualDisplayMode=StereoHW;
			// removin the -s
			str->TrimLeft("-s");
			str->TrimLeft();
		}

		// switch to Interlaced horizontal stereo mode
		if(str->Left(2)=="-i")
		{
			ActualDisplayMode=StereoInterlaced;
			// removin the -i
			str->TrimLeft("-i");
			str->TrimLeft();
		}

		// switch to Interlaced vertical stereo mode
		if(str->Left(2)=="-v")
		{
			ActualDisplayMode=StereoVertical;
			// removin the -v
			str->TrimLeft("-v");
			str->TrimLeft();
		}

		// if something left proces the file ...
		if(!str->IsEmpty()) 
		{
			int space_pos=str->Find(" ");

			if(space_pos==-1) // only one filename
			{
				ReadFouFile(*str,0);
			}
			else
			{
				CString file1=str->Left(space_pos);
				str->TrimLeft(file1); // first filename ...
				str->TrimLeft(); // second filename left in string ..
				ReadFouFile(file1,0);
				ReadFouFile(*str,1); 
			}
		}
	}
}
