// MCEView.h : interface of the CMCEView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MCEVIEW_H__1CF5FE6D_FA66_11D6_8A12_0050FC50B36D__INCLUDED_)
#define AFX_MCEVIEW_H__1CF5FE6D_FA66_11D6_8A12_0050FC50B36D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// Include the OpenGL headers
#include "gl\gl.h"
#include "gl\glu.h"
// Include other support headers
#include "MCEDataEngine.h"
#include "MCESupportRoutines.h"
#include "MCELevelControlPrimary.h"	// Added by ClassView
#include "MCELevelControlSecondary.h"	// Added by ClassView
#include "MCEPictureSetup.h"
#include "MCEGeneralSetup.h"

// non MCE useful headers
#include "Trackball.h"
#include "InterlaceStencil.h"

typedef enum tagDiplayMode {NoStereo,StereoHW,StereoInterlaced,StereoVertical}
DisplayMode;

typedef enum tageye{CentralEyePicking,CentralEye,LeftEye,RightEye}
Eye;

class CMCEView : public CView ,
				 CMCEDataEngine ,
				 CMCESupportRoutines,
				 CTrackball,
				 CInterlaceStencil
{
protected: // create from serialization only
	CMCEView();
	DECLARE_DYNCREATE(CMCEView)

// Attributes
public:
	CMCEDoc* GetDocument();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMCEView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CMCEView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	LRESULT OnApplyLevelControlChanges(WPARAM wParam, LPARAM lParam);
	LRESULT OnLevelControlExit(WPARAM wParam, LPARAM lParam);
	//{{AFX_MSG(CMCEView)
	afx_msg void OnFileOpen();
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDestroy();
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnCancelMode();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnViewLevelControl();
	afx_msg void OnFileSaveBmp();
	afx_msg void OnFileOpen2();
	afx_msg void OnViewLevelControlSec();
	afx_msg void OnEditAddnewpeak();
	afx_msg void OnEditClearpeaklist();
	afx_msg void OnPreferencesAnimation();
	afx_msg void OnUpdatePreferencesAnimation(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditAddnewpeak(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditClearpeaklist(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewLevelcontrol(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewLevelcontrolSec(CCmdUI* pCmdUI);
	afx_msg void OnFileSavepeaklist();
	afx_msg void OnUpdateFileSavepeaklist(CCmdUI* pCmdUI);
	afx_msg void OnPreferencesBackgroundcolor();
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnPreferencesGeneralSetup();
	afx_msg void OnFileSavepovrayinput();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

public:
	float m_fshiftY;
	float m_fshiftX;
	float m_fshiftZ;
	virtual BOOL SetupPixelFormat(void);
	virtual BOOL SetupViewport( int cx, int cy);
	virtual BOOL RenderScene (void);
	bool TestOpenGlError(const char * szMessage = "Sorry - no idea");

private:
	void ProcessCommandLine(CString* str);
	LRESULT OnLevelControlExitSec(WPARAM wParam, LPARAM lParam);
	LRESULT OnApplyLevelControlChangesSec(WPARAM wParam, LPARAM lParam);
	void SaveBMP(int Width, int Height);
	void Animate();
	void SupportInitializations();
	void RecalcModelView();
	void SetError (int e);
	BOOL InitializeOpenGL();
	void Camera(Eye mode);

	HGLRC m_hRC;
	CDC* m_pDC;
	DisplayMode ActualDisplayMode;

	static const char* const _ErrorStrings[];
	const char* m_ErrorString;

	// modeless dialog values
	CMCELevelControlPrimary* m_pDlgLevelControlPrim;
	CMCELevelControlSecondary* m_pDlgLevelControlSec;

	float stretchY;
	float stretchX;

	// mouse handling
	int m_beginy;
	int m_beginx;
	bool m_scaling;
	bool m_animate;
	bool m_moving;
	bool m_spinning;

	// trackbal support
	float curquat[4];
	float lastquat[4];
	int beginx, beginy;
	int W ,H;

	float scalefactor;

};

#ifndef _DEBUG  // debug version in MCEView.cpp
inline CMCEDoc* CMCEView::GetDocument()
   { return (CMCEDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MCEVIEW_H__1CF5FE6D_FA66_11D6_8A12_0050FC50B36D__INCLUDED_)
